################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
csvs2csv.py - join tables
=========================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script reads several tab-separated tables and joins them.

.. note:: 
   working with multiple columns per table and sorting is
   not implemented correctly and likely to fail.

Usage
-----

Example::

   python combine_tables.py --help

Type::

   python combine_tables.py --help

for command line help.

Documentation
-------------

Code
----

'''
import sys
import re
import string
import os
import time
import glob
import optparse

import CGAT.Experiment as E

##---------------------------------------------------------------------------------------------------------        
if __name__ == '__main__':

    parser = E.OptionParser( version = "%prog version: $Id: combine_tables.py 2782 2009-09-10 11:40:29Z andreas $", usage = globals()["__doc__"])

    parser.add_option("-t", "--no-titles", dest="titles", action="store_false",
                      help="no titles in input."  )

    parser.add_option("-i", "--skip-titles", dest="skip_titles", action="store_true",
                      help="skip output of titles."  )

    parser.add_option("-m", "--missing", dest="missing_value", type="string",
                      help="entry to use for missing values."  )

    parser.add_option( "--headers", dest="headers", type="string",
                      help="add headers for files."  )

    parser.add_option( "-c", "--columns", dest="columns", type="string",
                      help="columns to use for joining. Multiple columns can be specified as a comma-separated list [default=%default]."  )

    parser.add_option( "-g", "--glob", dest="glob", type="string",
                      help="wildcard expression for table names."  )

    parser.add_option("-s", "--sort", dest="sort", type="string",
                      help="sort by column titles alphabetical|numeric|list of columns." )

    parser.add_option("-e", "--merge", dest="merge", action="store_true",
                      help="simply merge tables without matching up rows. [default=%default]." )

    parser.add_option( "--sort-keys", dest="sort_keys", type="choice",
                      choices=("numeric", "alphabetic"),
                      help="sort key columns by value." )

    parser.add_option( "--keep-empty", dest="ignore_empty", action="store_false",
                      help="keep empty tables. The default is to ignore them." )

    parser.add_option( "--add-file-prefix", dest="add_file_prefix", action="store_true",
                      help="add file prefix to columns headers in multi-column tables [default=%default]" )

    parser.add_option( "--regex-filename", dest="regex_filename", type="string",
                      help="pattern to apply to filename to build prefix [default=%default]" )

    parser.set_defaults(
        titles = True,
        skip_titles = False,
        missing_value = "na",
        headers = None,
        sort = None,
        glob = None,
        columns = "1",
        sort_keys = False,
        merge = False,
        ignore_empty = True,
        add_file_prefix = False,
        regex_filename = "(.*)"
        )

    (options, args) = E.Start( parser )

    if options.headers: 
        if "," in options.headers:
            options.headers = options.headers.split(",")
        else:
            options.headers = re.split( "\s+", options.headers.strip() )

    if options.sort and options.sort not in ("numeric", "alphabetic"):
        if "," in options.sort:
            options.sort = options.sort.split( "," )
        else:
            options.sort = re.split("\s+", options.sort)

    if options.merge:
        options.columns = []
    else:
        options.columns = map(lambda x: int(x) - 1, options.columns.split(",") )

    options.filenames = []
    
    if options.glob:
        options.filenames += glob.glob( options.glob )
        
    options.filenames += args

    if len(options.filenames) < 1:
        print USAGE, "no tables specified/found."
        sys.exit(1)

    if options.loglevel >= 1:
        options.stdlog.write( "# combining %i tables.\n" % len(options.filenames) )
        sys.stdout.flush()
        if len(options.filenames) == 1:
            for line in open(options.filenames[0]):
                options.stdout.write( line )
            E.Stop()
            sys.exit(0)
        
    if options.headers and options.headers[0] != "auto" and \
       len(options.headers) != len(options.filenames):
        raise "number of provided headers (%i) is not equal to number filenames (%i)." %\
              (len(options.headers), len(options.filenames))

    tables = []
    keys = {}
    sorted_keys = []
    sizes = {}
    if options.merge:
        titles=["count"]
    else:
        titles = []
    
    for filename in options.filenames:

        prefix = os.path.basename( filename )

        if os.path.exists(filename):
            file = open(filename, "r")
            lines = filter( lambda x: x[0] <> "#", file)
            
        else:
            lines = []

        if len(lines) == 0 and options.ignore_empty: continue

        table = {}
        sizes = {}
        max_size = 0
        ncolumns = 0

        if options.titles:
            data = string.split(lines[0][:-1], "\t")
            if not titles:
                key = "-".join( [data[x] for x in options.columns] )                
                titles = [key]
            for x in range(len(data)):
                if x in options.columns: continue
                ncolumns += 1
                if options.add_file_prefix:
                    p = re.search( options.regex_filename, prefix).groups()[0]
                    titles.append( "%s_%s" % ( p, data[x] ) )
                else:
                    titles.append( data[x] )
                
            del lines[0]
        else:
            ncolumns = 1

        n = 0
        for line in lines:
            data = string.split(line[:-1], "\t")
            row_keys = [ data[x] for x in options.columns ]
            if options.sort_keys: 
                if options.sort_keys == "numeric":
                    row_keys.sort( lambda x,y: cmp(float(x), float(y)) )
                else:
                    row_keys.sort()
            if options.merge:
                key = n
            else:
                key = "-".join( row_keys ) 

            if not keys.has_key(key):
                sorted_keys.append( key )
                keys[key] = 1
                sizes[key] = 0

            max_size = max( len(data) - len(options.columns), max_size )
            table[key] = [ data[x] for x in filter( lambda x: x not in options.columns, range(0,len(data)) )]
            n += 1

        ## enter columns of "na" for empty tables.
        if max_size == 0: max_size = ncolumns

        tables.append( (max_size, table) )

    if len(tables) == len(titles) - 1:
        
        if options.headers:
            headers = ["bin"]
            if options.headers[0] == 'auto':
                for t in range(len(tables)):
                    headers.append(os.path.basename(options.filenames[t]))
                    headers += [""] * (tables[t][0] - 1)

            else:
                for t in range(len(tables)):
                    headers.append(options.headers[t])
                    headers += [""] * (tables[t][0] - 1)                

            ## use headers as titles, if headers is given and skip-titles is turned on
            if options.titles and options.skip_titles:
                titles = headers
            else:
            ## otherwise: print the headers out right away            
                sys.stdout.write( string.join( headers, "\t" ) + "\n")

        order = range(0, len(tables)+1)

        if options.titles:

            if options.sort:
                sort_order = []

                if options.sort == "numeric":
                    t = zip( map(int, titles[1:]), range( 1, len(titles) + 1) )
                    t.sort()

                    for tt in t:
                        sort_order.append( titles[tt[1]] )

                elif options.sort == "alphabetical":
                    t = zip( titles[1:], range( 1, len(titles) + 1) )
                    t.sort()

                    for tt in t:
                        sort_order.append( titles[tt[1]] )
                else:
                    sort_order = options.sort

                map_title2pos = {}
                for x in range(1, len(titles)):
                    map_title2pos[titles[x]] = x

                order = [0,]
                for x in sort_order:
                    if x in map_title2pos:                
                        order.append( map_title2pos[x] )

            else:
                order = range(0, len(titles) )

            sys.stdout.write( "\t".join( map(lambda x: titles[order[x]], range(len(titles)))))
            sys.stdout.write("\n")

        if options.sort_keys:
            if options.sort_keys: 
                if options.sort_keys == "numeric":
                    sorted_keys.sort( lambda x,y: cmp(float(x), float(y)) )
                else:
                    sorted_keys.sort()

        for key in sorted_keys:

            sys.stdout.write( "%s" % key)

            for x in order[1:]:
                max_size, table = tables[x-1]
                c = 0
                if table.has_key( key ):
                    sys.stdout.write("\t")
                    sys.stdout.write( string.join(table[key], "\t") )
                    c = len(table[key])

                assert(max_size == 1)

                sys.stdout.write( "\t%s" % options.missing_value * (max_size - c) )

            sys.stdout.write("\n")

    else:


        # for multi-column table, just write
        if options.titles:
            sys.stdout.write( "\t".join( map(lambda x: titles[x], range(len(titles)))))
            sys.stdout.write("\n")

        for key in sorted_keys:
            
            sys.stdout.write( "%s" % key)

            for x in range(len(tables)):
            
                max_size, table = tables[x]
                c = 0
                if table.has_key( key ):
                    sys.stdout.write("\t")
                    sys.stdout.write( "\t".join(table[key]) )
                    c = len(table[key])

                sys.stdout.write( "\t%s" % options.missing_value * (max_size - c) )

            sys.stdout.write("\n")
    
    E.Stop()





################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
AString.py - a compact string of characters
===========================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Code
----

'''
import array

##------------------------------------------------------------
class AString:
    """an array posing as a sequence.

    This class conserves memory as it uses only 1 byte per letter,
    while python strings use the machine word size for a letter.

    It exports a mixture of the methods in the python string and 
    python array classes.

    .. note::

       Using this class will incur a heavy penalty compared to
       using :class:array.array directly. Only use sparingly for
       heavy computations.
    """
    
    def __init__(self, *args):
        self.mArray = array.array("c", *args )

    def __getitem__(self, *args ):
        return self.mArray.__getitem__( *args )

    def upper( self ):
        """return upper case version."""
        return AString( self.mArray.tostring().upper() )
    
    def lower(self):
        """return lower case version."""
        return AString( self.mArray.tostring().lower() )

    def insert( self, *args ):
        self.mArray.insert( *args )
        
    def index( self, *args ):
        return self.mArray.index( *args )

    def count( self, *args ):
        return self.mArray.count( *args )

    def reverse( self, *args ):
        self.mArray.reverse( *args )

    def extend( self, *args ):
        self.mArray.extend( *args )

    def remove( self, *args ):
        self.mArray.remove( *args )

    def fromstring( self, *args ):
        self.mArray.fromstring( *args )

    def tostring( self, *args ):
        return self.mArray.tostring( *args )

    def __getslice__( self, *args):
        """return slice as a string."""
        return self.mArray.__getslice__(*args).tostring()

    def __setslice__( self, start, end, sub):
        """set slice start:end from a string sub."""
        return self.mArray.__setslice__(start, end,
                                        array.array("c", sub ))

    def __str__(self):
        return self.mArray.tostring()

    def __getattr__(self, name):
        return getattr( self.mArray, name)



#! /bin/env python
################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
CSV2DB.py - utilities for uploading a table to database
=======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

create a table from a csv separated file and load data into it.

This module supports backends for postgres and sqlite3. Column types are
auto-detected.

.. todo::

   Use file import where appropriate to speed up loading. Currently, this is
   not always the case.

Usage
-----

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import time
import optparse
import tempfile
import subprocess
import types

import Experiment as E
import csv
import CSV as CSV
import sqlite3

def executewait( dbhandle, statement, error, retry = False, wait=5):
    '''execute sql statement.

    Retry on error, if retry is True.
    Returns a cursor object.
    '''

    cc = dbhandle.cursor()    
    i = 20
    while i>0:
        try:
            cc.execute( statement )
            return cc
        except sqlite3.OperationalError as e:
            msg = e.message
            E.warn("import failed: msg=%s, statement=\n  %s" % (msg, statement ) )
        # TODO: check for database locked msg
            if not retry:
                raise error, msg
            if not re.search("locked", str(msg)):
                raise error, msg
            time.sleep(wait)
            i -= 1
            continue
        break
    raise sqlite.OperationalError("Database locked and too many retries")

def quoteRow( row, take, 
              map_column2type, 
              missing_values, 
              null = "NULL",
              string_value = "%s" ):
    """return a dictionary with properly quoted values."""

    # set empty values for int/float to NULL
    d = {}
    for t in take:
        v = row[t]
        if v == "": 
            d[t] = null
        elif v in missing_values: 
            d[t] = null
        elif map_column2type[t] in (types.IntType, types.FloatType):
            d[t] = str(row[t])
        else:
            d[t] = string_value % row[t]
            
    return d

def quoteTableName( name, quote_char = "_", backend="sqlite" ):
    if backend == "sqlite":
        # no special characters. Column names can not start with a number.
        if name[0] in "0123456789": name = "_" + name
        return re.sub( "[-(),\[\].:]", "_", name )
    elif backend in ("mysql","pg"):
        if name[0] in "0123456789": name = "_" + name
        return re.sub( "[-(),\[\]]:", "_", name )

def createTable( dbhandle, error, options, rows = None, headers = None,
                 existing_tables = []):

    ## create table by guessing column types from data type.
    if rows:
        map_column2type, ignored, max_values = CSV.GetMapColumn2Type( rows,
                                                                      ignore_empty = options.ignore_empty,
                                                                      get_max_values = True )
        if ignored:
            E.info( "ignored columns: %s" % str(ignored) )

        headers = map_column2type.keys()
        headers.sort()

    elif headers:
        map_column2type = dict( zip( headers, [None,] * len(headers) ) )
        ignored = 0

    columns_to_ignore = set( [ x.lower() for x in options.ignore_columns] )
    columns_to_rename = dict( [x.lower().split(":") for x in options.rename_columns ] )

    take = []
    ## associate headers to field names
    columns = []
    present = {}
    for h in headers:
        hh = h
        if options.lowercase:
            hh = string.lower(h)

        if hh in columns_to_ignore: continue
    
        if hh in present:
            if options.ignore_duplicates:
                continue
            else:
                raise ValueError("duplicate column %s" % hh)

        present[hh] = 1
        take.append( h )

        if map_column2type[h] == types.IntType:
            max_value = max_values[h]
            if h > 2147483647:
                t = "BIGINT DEFAULT '0'"
            elif h > 32767:
                t = "INTEGER DEFAULT '0'"
            else:
                t = "SMALLINT DEFAULT '0'"

        elif map_column2type[h] == types.FloatType:
            t = "FLOAT DEFAULT '0'"
        else:
            t = "TEXT"

        # remove special characters from column names
        if hh == "": raise ValueError("column '%s' without header " % h )
        hh = columns_to_rename.get( hh, hh )
        hh = re.sub( '''['"]''', "", hh)
        hh = re.sub( "[,;.:\-\+/ ()%]", "_", hh)
        if hh[0] in "0123456789": hh = "_" + hh
        columns.append( "%s %s" % (hh, t))

    # delete old table if it exists
    while 1:
        try:
            cc = dbhandle.cursor()
            cc.execute("DROP TABLE IF EXISTS '%s'" % options.tablename)
            dbhandle.commit()
            cc.close()
            E.info("existing table %s deleted" % options.tablename )
        except sqlite3.OperationalError, msg:
            E.warn( msg )
            time.sleep(5)
            continue
        except error, msg:
            E.warn( "could not delete existing table %s: %s" % (options.tablename, str(msg) ))
            dbhandle.rollback()
            if not options.retry:
                raise error, msg
            elif options.tablename in existing_tables:
                # table exists, but drop did not work (e.g. database lock)
                time.sleep(5)
                continue
            else:
                # table might not have existed
                break
        break
    
    ## create new table
    statement = "CREATE TABLE %s ( %s );" % (options.tablename, ", ".join( columns))

    E.debug( "table create:\n# %s" % (statement ) )
        
    while 1:
        try:
            cc = dbhandle.cursor()
            cc.execute(statement)
            cc.close()
            dbhandle.commit()
        except error, msg:
            E.warn("table creation failed: msg=%s, statement=\n  %s" % (msg, statement ) )
            # TODO: check for database locked msg
            if not options.retry:
                raise error, msg
            if not re.search("locked", str(msg)):
                raise error( "%s: %s" % (msg, statement))
            time.sleep(5)
            continue
        break

    E.info("table %s created successfully." % options.tablename )
    
    return take, map_column2type, ignored

def run( infile, options ):

    options.tablename = quoteTableName( options.tablename, backend = options.backend )

    if options.map:
        m = {}
        for x in options.map:
            f,t = x.split(":")
            m[f] = t
        options.map = m
    else:
        options.map = {}
        
    existing_tables = None
    
    if options.backend == "pg":
        import pgdb
        dbhandle = pgdb.connect( options.psql_connection )
        error = pgdb.DatabaseError
        options.null = "NULL"
        options.string_value = "'%s'"
        if options.insert_quick:
            raise ValueError("quick import not implemented.")

    elif options.backend == "sqlite":
        import sqlite3
        dbhandle = sqlite3.connect( options.database )
        try:
            os.chmod( options.database, 0664 )
        except OSError, msg:
            E.warn("could not change permissions of database: %s" % msg )

        # Avoid the following error:
        # sqlite3.ProgrammingError: You must not use 8-bit bytestrings unless you use a text_factory that can interpret 8-bit bytestrings (like text_factory = str). It is highly recommended that you instead just switch your application to Unicode strings
        # Note: might be better to make csv2db unicode aware.
        dbhandle.text_factory = str

        error = sqlite3.OperationalError
        options.insert_many = True  # False
        options.null = None # "NULL" 
        options.string_value = "%s" # "'%s'"

        statement = "SELECT name FROM sqlite_master WHERE type='table'"
        cc = executewait( dbhandle, statement, error, options.retry )
        existing_tables = set( [ x[0] for x in cc ] )
        cc.close()

        quick_import_statement = "sqlite3 -header -csv -separator '\t' %s '.import %%s %s'" % (options.database, options.tablename)

    if options.header != None:
        options.header = [x.strip() for x in options.header.split(",")]

    if options.utf:
        reader = CSV.UnicodeDictReader( infile, 
                             dialect=options.dialect, 
                             fieldnames = options.header )
    else:
        reader = CSV.DictReader( infile, 
                                 dialect=options.dialect, 
                                 fieldnames = options.header )

    if options.replace_header:
        reader.next()

    E.info("reading %i columns to guess column types" % options.guess_size)

    rows = []
    for row in reader:
        if None in row: 
            raise ValueError( "undefined columns in input file at row: %s" % row )
        
        try:
            rows.append( CSV.ConvertDictionary( row , map=options.map ))
        except TypeError, msg:
            E.warn( "incomplete line? Type error in conversion: '%s' with data: %s" % (msg, str(row) ) )
        except ValueError, msg:
            E.warn( "incomplete line? Type error in conversion: '%s' with data: %s" % (msg, str(row) ) )

        if len(rows) >= options.guess_size:
            break

    E.info("read %i rows for type guessing" % len(rows) )
    E.info("creating table" )

    if len(rows) == 0:
        if options.allow_empty:
            if not reader.fieldnames:
                E.warn( "no data - no table created")
            else:
                # create empty table and exit
                take, map_column2type, ignored = createTable( dbhandle, error, headers = reader.fieldnames,
                                                              options = options,
                                                              existing_tables = existing_tables )
                E.info( "empty table created" )
            return
        else:
            raise ValueError( "empty table" )
    else:
        take, map_column2type, ignored = createTable( dbhandle, error,
                                                      rows = rows,
                                                      options = options,
                                                      existing_tables = existing_tables )


    def row_iter( rows, reader):
        for row in rows: 
            yield quoteRow( row, take, map_column2type, 
                            options.missing_values, 
                            null = options.null,
                            string_value = options.string_value )
        for data in reader: 
            yield quoteRow( CSV.ConvertDictionary( data , map=options.map ), take, map_column2type, 
                            options.missing_values, 
                            null = options.null,
                            string_value = options.string_value )

    ninput = 0

    E.info("inserting data")

    if options.insert_quick:
        E.info( "using quick insert" )

        outfile, filename = tempfile.mkstemp()

        E.debug("dumping data into %s" % filename )

        for d in row_iter( rows, reader ):

            ninput += 1
            os.write( outfile, "\t".join( [ str(d[x]) for x in take ] ) + "\n" )

            if options.loglevel >= 1 and ninput % options.report_step == 0:
                options.stdlog.write( "# iteration %i\n" % ninput )
                options.stdlog.flush()

        os.close( outfile )

        statement = quick_import_statement % filename
        E.debug( statement )

        # infinite loop possible
        while 1:
            
            retcode = E.run( statement, cwd = os.getcwd(), close_fds = True)
        
            if retcode != 0:
                E.warn("import error using statement: %s" % statement)

                if not options.retry:
                    raise ValueError( "import error using statement: %s" % statement )
            
                time.sleep(5)
                continue

            break
        
        os.remove( filename )
        
        # there is no way to .insert NULL values into sqlite. The only solution is
        # to update all colums.
        for column in take:
            executewait( dbhandle, 
                         "UPDATE %s SET %s = NULL WHERE %s = 'None'" % (options.tablename, column, column),
                         error, 
                         options.retry)

    elif options.insert_many:
        data = []
        for d in row_iter( rows, reader ):
            ninput += 1

            data.append( [d[x] for x in take] )

            if ninput % options.report_step == 0:
                E.info( "iteration %i" % ninput )
                
        statement = "INSERT INTO %s VALUES (%s)" % (options.tablename, ",".join( "?" * len(take ) ) )

        E.info( "inserting %i rows" % len(data) )
        E.debug( "multiple insert:\n# %s" % statement )

        while 1:
            try:
                dbhandle.executemany( statement, data )
            except error, msg:
                E.warn("import failed: msg=%s, statement=\n  %s" % (msg, statement ) )
                # TODO: check for database locked msg
                if not options.retry:
                    raise error, msg
                if not re.search("locked", str(msg)):
                    raise error, msg
                time.sleep(5)
                continue
            break


    else:
        ## insert line by line (could not figure out how to do bulk loading with subprocess and COPY FROM STDIN)
        statement = "INSERT INTO %s VALUES (%%(%s)s)" % (options.tablename,
                                                         ')s, %('.join( take ))

        # output data used for guessing:
        for d in row_iter( rows, reader):

            ninput += 1

            E.debug( "single insert:\n# %s" % (statement % d) )
            cc = executewait( dbhandle, statement % d, error, options.retry )
            cc.close()
            
            if options.loglevel >= 1 and ninput % options.report_step == 0:
                E.info("iteration %i" % ninput )

    E.info("building indices")
    nindex = 0
    for index in options.indices:
        
        nindex += 1
        try:
            statement = "CREATE INDEX %s_index%i ON %s (%s)" % ( options.tablename, nindex, options.tablename, index )
            cc = executewait( dbhandle, statement, error, options.retry )
            cc.close()
            E.info( "added index on column %s" % (index) )
        except error, msg: 
            E.info( "adding index on column %s failed: %s" % (index, msg) )

    statement = "SELECT COUNT(*) FROM %s" % ( options.tablename )
    cc = executewait( dbhandle, statement, error, options.retry )
    result = cc.fetchone()
    cc.close()

    noutput = result[0]
    
    E.info("ninput=%i, noutput=%i, nskipped_columns=%i" % (ninput, noutput, len(ignored)) )

    dbhandle.commit()

def buildParser( ):

    parser = E.OptionParser( version = "%prog version: $Id: csv2db.py 2782 2009-09-10 11:40:29Z andreas $", usage = globals()["__doc__"])

    parser.add_option( "--dialect", dest="dialect", type="string",
                      help="csv dialect to use [default=%default]." )

    parser.add_option("-m", "--map", dest="map", type="string", action="append",
                      help="explicit mapping function for columns The format is column:type (e.g.: length:int) [default=%default]." )

    parser.add_option("-t", "--table", dest="tablename", type="string",
                      help="table name for all backends [default=%default]." )

    parser.add_option("-d", "--database", dest="database", type="string",
                      help="database name for sqlite3 [default=%default]." )

    parser.add_option("-H", "--header", dest="header", type="string",
                      help="',' separated list of header for files without header [default=%default]." )

    parser.add_option( "--replace-header", dest="replace_header", action="store_true",
                       help="replace header with --header instead of adding it [default=%default]." )

    parser.add_option("-l", "--lowercase", dest="lowercase", action="store_true",
                      help="force lower case column names [default=%default]." )

    parser.add_option("-u", "--ignore-duplicates", dest="ignore_duplicates", action="store_true",
                      help="ignore columns with duplicate names [default=%default]." )

    parser.add_option("-s", "--ignore-same", dest="ignore_same", action="store_true",
                      help="ignore columns with identical values [default=%default]." )

    parser.add_option("--ignore-column", dest="ignore_columns", type="string", action="append",
                      help="ignore columns [default=%default]." )

    parser.add_option("--rename-column", dest="rename_columns", type="string", action="append",
                      help="rename columns [default=%default]." )
    
    parser.add_option("-e", "--ignore-empty", dest="ignore_empty", action="store_true",
                      help="ignore columns which are all empty [default=%default]." )

    parser.add_option("-q", "--quick", dest="insert_quick", action="store_true",
                      help="try quick file based import - needs to be supported by the backend [default=%default]." )

    parser.add_option("-b", "--backend", dest="backend", type="choice",
                      choices=("pg", "sqlite"),
                      help="database backend to choose [default=%default]." )

    parser.add_option("-i", "--index", dest="indices", type="string", action="append",
                      help="create an index for the named column [default=%default]." )

    parser.add_option("-a", "--allow-empty", dest="allow_empty", action="store_true",
                      help="allow empty table [default=%default]." )

    parser.add_option( "--retry", dest="retry", action="store_true",
                      help="retry if an SQL statement fails - warning: THIS MIGHT CAUSE DEADLOCKS [default=%default]." )

    parser.add_option( "-z", "--from-zipped", dest="from_zipped", action="store_true",
                       help="input is zipped.")
    parser.add_option( "--utf8", dest="utf", action="store_true",
                       help="standard in is encoded as UTF8 rather than local default, WARNING: does not strip comment lines yet [default=%default]")

    parser.set_defaults(
        map = [],
        dialect = "excel-tab",
        database = "csvdb",
        lowercase = False,
        tablename = "csv",
        from_zipped = False,
        ignore_duplicates= False,
        ignore_identical = False,
        ignore_empty = False,
        insert_many = False,
        ignore_columns = [],
        rename_columns = [],
        header = None,
        replace_header = False,
        guess_size = 1000,
        report_step = 10000,
        backend="sqlite",
        indices = [],
        missing_values = ("na", "NA", ),
        insert_quick = False,
        allow_empty = False,
        retry = False,
        utr = False
        )
    
    return parser



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
peptides2cds.py - map peptide sequences onto nucleic acid sequences
===================================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

build a map between peptides to cdnas. This script deals with
frameshifts but not with larger indels. 

This script requires the :mod:`alignlib` library and :file:`muscle`
to be installed.

Usage
-----

Example::

   python peptides2cds.py --help

Type::

   python peptides2cds.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import tempfile
import time
import optparse
import CGAT.Experiment as E
import CGAT.BlastAlignments as BlastAlignments
import alignlib
import CGAT.Genomics as Genomics
import CGAT.FastaIterator as FastaIterator
import CGAT.IOTools as IOTools

def AlignExhaustive( seq_wobble, seq_cds, seq_peptide, map_p2c, options, diag_width = 2 ):
    """Align two sequences.

    Align in chunks to keep memory low. Both sequences are roughly the same,
    thus align only in diagonal.
    """

    gop, gep = -1.0, -1.0
    matrix = alignlib.makeSubstitutionMatrixBackTranslation( 1, -10, 1, alignlib.getDefaultEncoder() )
    alignlib.setDefaultSubstitutionMatrix( matrix )

    if seq_wobble.getLength() < 10000:
        if options.loglevel >= 6:
            options.stdlog.write( "# using full dynamic programing matrix.\n" )
            options.stdlog.flush()
        # do not penalize gaps at the end, because sometimes the last codon might be missing
        alignator = alignlib.makeAlignatorDPFull( alignlib.ALIGNMENT_GLOBAL, gop, gep, 1, 1 )
    else:
        diag_width = abs(seq_wobble.getLength() - seq_cds.getLength()) + 1
        if options.loglevel >= 6:
            options.stdlog.write( "# using dot alignment with diagonal %i\n" % diag_width )
            options.stdlog.flush()

        dots = alignlib.makeAlignmentMatrixRow()
            
        for x in range(0, seq_wobble.getLength()):
            xr = seq_wobble.asResidue( x )
            for y in range( max(0, x - diag_width), min( seq_cds.getLength(), x + diag_width)):
                s = matrix.getValue( xr, seq_cds.asResidue(y) )
                if s >= 0:
                    dots.addPair( x, y, float(s) )

        if options.loglevel >= 6:
            options.stdlog.write( "# finished adding %i dots" % dots.getLength() )
            options.stdlog.flush()

        alignator_dummy = alignlib.makeAlignatorPrebuilt( dots )
        
        alignator = alignlib.makeAlignatorDots( alignator_dummy, gop, gep )

    alignator.align( map_p2c, seq_wobble, seq_cds )

def FindMatch( matrix, seq1, x, seq2, y, options, max_advance = 10, min_match = 3 ):
    """advance on sequence2 from position y until you find a minimum match of
    min_match residues.

    returns the offset in y. If no match is found, an offset of 0 is returned.
    """

    #for z in range(0,10):
    #print seq1.asChar(x+z), seq2.asChar(y+z)

    min_match = min( seq1.getLength() - x, min_match )
    l2 = seq2.getLength()
    # get list of reference residues
    rr = []
    for xx in range(x, x + min_match ):
        rr.append( seq1.asResidue(xx) )

    max_advance = min( max_advance, l2 - y)
    for yy in range(y, y + max_advance):
        m = True
        for i in range( 0, min_match ):
            if matrix.getValue( rr[i], seq2.asResidue(yy + i) ) < 0:
                m = False
                break
        if m: return yy - y

    return 0

def AlignCodonBased( seq_wobble, seq_cds, seq_peptide, map_p2c, options,
                     diag_width = 2, max_advance = 2 ):
    """advance in codons in seq_wobble and match to nucleotides in seq_cds.

    Due to alinglib this is all in one-based coordinates.
    Takes care of frameshifts.
    """
    
    map_p2c.clear()

    gop, gep = -1.0, -1.0
    matrix = alignlib.makeSubstitutionMatrixBackTranslation( 1, -10, 1, alignlib.getDefaultEncoder() )

    pep_seq = seq_peptide.asString()
    cds_seq = seq_cds.asString()
    wobble_seq = seq_wobble.asString()
    
    lcds = seq_cds.getLength()
    lwobble = seq_wobble.getLength()
    y = 0
    x = 0

    last_start = None

    while x < lwobble and y < lcds:

        xr = seq_wobble.asResidue( x )
        # skip over masked chars in wobble - these are gaps
        if seq_wobble.asChar(x) == "X": 
            x += 1
            continue

        # skip over masked chars in wobble - these are from
        # masked chars in the peptide sequence
        # Note to self: do not see all implications of this change
        # check later.
        if seq_wobble.asChar(x) == "N": 
            x += 1
            continue

        # skip over gaps in wobble 
        if seq_wobble.asChar(x) == "-": 
            x += 1
            continue

        s = matrix.getValue( xr, seq_cds.asResidue(y) )

        if options.loglevel >= 6:
            if (x % 3 == 0):
                c = seq_cds.asChar(y) + seq_cds.asChar(y+1) + seq_cds.asChar(y+2)
                options.stdlog.write( "# c=%s, x=%i, y=%i, aa=%s target=%s\n" % (c, x, y,
                                                                                 Genomics.MapCodon2AA( c ),
                                                                                 pep_seq[int(x/3)]) )
                                      
            options.stdlog.write( "# x=%i\twob=%s\ty=%i\tcds=%s\txr=%s\tcds=%i\tscore=%s\n" % \
                                      (x, seq_wobble.asChar(x), y, seq_cds.asChar(y), xr, seq_cds.asResidue(y), str(s) ))
            
        # deal with mismatches
        if s <= 0:

            tmp_map_p2c = alignlib.makeAlignmentVector()

            ## backtrack to previous three codons and align
            ## three codons for double frameshifts that span two codons and
            ## produce two X's and six WWWWWW.

            ## number of nucleotides to extend (should be multiple of 3)
            ## less than 12 caused failure for some peptides.
            d = 15
            
            # extend by amound dx
            dx = (x % 3) + d
            
            x_start = max(0, x - dx )
            # map to ensure that no ambiguous residue mappings
            # exist after re-alignment
            y_start = max(0, map_p2c.mapRowToCol( x_start, alignlib.RIGHT ))

            if (x_start, y_start) == last_start:
                raise ValueError( "infinite loop detected" )

            last_start = (x_start, y_start)

            x_end = min(x_start + 2 * d, len(wobble_seq) )
            y_end = min(y_start + 2 * d, len(cds_seq) )

            wobble_fragment = alignlib.makeSequence(wobble_seq[x_start:x_end])
            cds_fragment = alignlib.makeSequence(cds_seq[y_start:y_end])
            
            AlignExhaustive( wobble_fragment, cds_fragment, "", tmp_map_p2c, options )

            if options.loglevel >= 10:
                 options.stdlog.write("# fragmented alignment from %i-%i, %i-%i:\n%s\n" % (x_start, x_end,
                                                                                           y_start, y_end,
                                                                                           str(alignlib.AlignmentFormatExplicit( tmp_map_p2c,
                                                                                                                                 wobble_fragment, 
                                                                                                                                 cds_fragment ))))
                 
                 options.stdlog.flush()

            ## clear alignment
            map_p2c.removeRowRegion( x_start, x_end )
            ngap = 0
            last_x, last_y = None, None
            for xxx in range( tmp_map_p2c.getRowFrom(), tmp_map_p2c.getRowTo() ):
                yyy = tmp_map_p2c.mapRowToCol(xxx)

                if yyy >= 0:
                    x = xxx + x_start
                    y = yyy + y_start
                    xr = seq_wobble.asResidue(x)
                    s = matrix.getValue( seq_wobble.asResidue(x), seq_cds.asResidue(y) )
                    if s < 0:
                        raise ValueError("mismatched residue wobble: %i (%s), cds: %i (%s)" % (x, seq_wobble.asChar(x), y, seq_cds.asChar(y)))
                    
                    map_p2c.addPair( x, y, s)
                    last_x, last_y = x, y
                    if options.loglevel >= 6:
                        options.stdlog.write( "# reset: x=%i\twob=%s\ty=%i\tcds=%s\txr=%s\tcds=%i\tscore=%i\n" % \
                                              (x, seq_wobble.asChar(x), y, seq_cds.asChar(y), xr, seq_cds.asResidue(y), s ))
                        options.stdlog.flush()
                    ngap = 0
                else:
                    ngap += 1

                # treat special case of double frameshifts. They might cause a petide/wobble residue
                # to be eliminated and thus the translated sequences will differ.
                # simply delete the last residue between x and y and move to next codon.
                if ngap == 3:
                    map_p2c.removeRowRegion( last_x, last_x + 1 )

                    last_x += 1
                    map_p2c.addPair( last_x, last_y )
                    if options.loglevel >= 6:
                        options.stdlog.write( "# double: x=%i\twob=%s\ty=%i\tcds=%s\txr=%s\tcds=%i\tscore=%i\n" % \
                                              (last_x, seq_wobble.asChar(last_x), last_y, seq_cds.asChar(last_y), xr, seq_cds.asResidue(last_y), s ))
                        options.stdlog.flush()                    
                    ngap = 0
                    
            ## exit condition if alignment is shorter than problematic residue
            ## need to catch this to avoid infinite loop.
            if tmp_map_p2c.getRowTo() < d:
                if lwobble - x <= 4:
                    ## only last codon is missing, so ok
                    break
                else:
                    raise ValueError("failure to align in designated window.")
                    
            s = 0
            
        s = matrix.getValue( xr, seq_cds.asResidue(y) )

        if s < 0:
            raise ValueError("mis-matching residues.")
        
        map_p2c.addPair( x, y, float(s) )
        
        # advance to next residues
        x += 1
        y += 1

    # sanity checks
    assert( map_p2c.getRowTo() <= seq_wobble.getLength() )
    assert( map_p2c.getColTo() <= seq_cds.getLength() )

def PrintPrettyAlignment( seq_wobble, seq_cds, seq_pep, map_p2c, options ):
    """print a pretty alignment."""

    f = alignlib.AlignmentFormatExplicit( map_p2c, seq_wobble, seq_cds )
    wobble_ali, cds_ali = f.mRowAlignment, f.mColAlignment
    
    wi, ci, pi = 0, 0, 0
    frags_w, frags_c, frags_p = [], [], []
    for x in range( 0, len( wobble_ali )):

        if wi % 3 == 0:
            if pi < len(seq_pep):
                frags_p.append( "  %s " % seq_pep[pi])
            frags_w.append (" ")
            frags_c.append (" ")
            pi += 1

        frags_w.append( wobble_ali[x] )
        frags_c.append( cds_ali[x] )
        if wobble_ali[x] != "-":
            wi += 1
            
        if len(frags_w) > 120 and len(frags_w) % 3 == 0:
            options.stdlog.write( "#" + "".join(frags_w) + "\n" )
            options.stdlog.write( "#" + "".join(frags_p) + "\n" )
            options.stdlog.write( "#" + "".join(frags_c) + "\n" )                
            options.stdlog.write( "#\n" )
            frags_w, frags_c, frags_p = [], [], []


    options.stdlog.write( "#" + "".join(frags_w) + "\n" )
    options.stdlog.write( "#" + "".join(frags_p) + "\n" )
    options.stdlog.write( "#" + "".join(frags_c) + "\n" )                
    options.stdlog.write( "#\n" )
    

##--------------------------------------------------------------------------
##--------------------------------------------------------------------------
##--------------------------------------------------------------------------
##--------------------------------------------------------------------------        
def getMapPeptide2Cds( peptide_sequence, cds_sequence, options ):
    """get map between peptide sequence and cds sequence.
    
    The returned alignment is in nucleotides.

    """
    
    ## remove whitespaces form protein sequence
    p = re.sub(" ", "", peptide_sequence )

    ## remove gaps and whitespaces from cds
    c = re.sub("[ .-]", "", cds_sequence )

    w = Genomics.Protein2Wobble( p.upper() )

    if options.loglevel >= 6:
        options.stdlog.write( "# peptide original (%5i): %s\n" % (len(p), p) )
        options.stdlog.write( "# cds original     (%5i): %s\n" % (len(c), c) )
        options.stdlog.write( "# wobble sequence  (%5i): %s\n" % (len(w), w) )
        options.stdlog.flush()

    seq_wobble = alignlib.makeSequence( w )
    seq_cds = alignlib.makeSequence( string.upper(c) )
    seq_peptide = alignlib.makeSequence( p )

    map_p2c = alignlib.makeAlignmentVector()

    try:
        AlignCodonBased( seq_wobble, seq_cds, seq_peptide, map_p2c, options = options )
    except ValueError, msg:
        raise ValueError( "mapping error for sequence: %s" % (msg) )

    ## if there are more than five frameshifts - do exhaustive alignment
    max_gaps = 5
    num_peptide_gaps = len( re.sub("[^-]", "", p ) )
    ngaps = map_p2c.getNumGaps() - (num_peptide_gaps *  3) - abs(len(w)-len(c))
    
    if options.loglevel >= 6:
        options.stdlog.write("# alignment between wobble and cds: ngaps=%i, npeptide_gaps=%i\n" % (ngaps, num_peptide_gaps) )
        PrintPrettyAlignment( seq_wobble, seq_cds, p, map_p2c, options )

    if ngaps > max_gaps:
        if options.loglevel >= 2:
            options.stdlog.write("# too many gaps (%i>%i), realigning exhaustively.\n" % (ngaps, max_gaps ) )
            options.stdlog.flush()
        full_map_p2c = alignlib.makeAlignmentVector()
        
        AlignExhaustive( seq_wobble, seq_cds, seq_peptide, full_map_p2c, options )
        if options.loglevel >= 6:
            options.stdlog.write("# full alignment between wobble and cds:\n" )
            options.stdlog.flush()
            PrintPrettyAlignment( seq_wobble, seq_cds, p, full_map_p2c, options )

        map_p2c = full_map_p2c
        
    ## remove incomplete codons
    x = 0
    while x < len(p) * 3:
        if (map_p2c.mapRowToCol( x ) < 0 or \
            map_p2c.mapRowToCol( x+1 ) < 0 or \
            map_p2c.mapRowToCol( x+2 ) < 0 ):
            map_p2c.removeRowRegion( x, x + 3)
        x += 3

    if map_p2c.getLength() == 0:
        if options.loglevel >= 1:
            options.stdlog.write("# WARNING: empty alignment\n" )
            if options.loglevel >= 6:
                options.stdlog.write( "# peptide original: %s\n" % p )
                options.stdlog.write( "# cds original    : %s\n" % c )
                options.stdlog.write( "# wobble sequence : %s\n" % w )

        raise ValueError( "empty alignment" )

    assert( map_p2c.getRowTo() <= seq_wobble.getLength() )
    assert( map_p2c.getColTo() <= seq_cds.getLength() )

    return map_p2c
    
        
if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: peptides2cds.py 2890 2010-04-07 08:58:54Z andreas $")

    parser.add_option("-p", "--peptides", dest="filename_peptides", type="string",
                      help="filename with peptide sequences [%default]."  )
    
    parser.add_option("-c", "--cds", "--cdnas", dest="filename_cdna", type="string",
                      help="filename with cdna sequences [%default]."  )

    parser.add_option("-m", "--map", dest="filename_map", type="string",
                      help="filename with map of peptide identifiers to cdna identifiers [%default]."  )

    parser.add_option( "--output-identifier", dest="output_identifier", type="choice",
                       choices=("cdna", "peptide"),
                       help="output identifier to use [%default]."  )


    parser.add_option("-f", "--output-format=", dest="output_format", type="choice",
                      choices=("alignment", "fasta"),
                      help="output format.")
    
    parser.set_defaults(
        peptides=None,
        filename_cdna = None,
        output_format="alignment",
        filename_map = None,
        stop_codons = ("TAG", "TAA", "TGA"),
        output_identifier = "peptide",
        )

    (options, args) = E.Start( parser, add_pipe_options = True )

    if not options.filename_cdna:
        raise ValueError("please supply filename with cds sequences.")

    if options.filename_peptides:
        infile = open(options.filename_peptides, "r") 
        E.info("reading from %s" % options.filename_peptides)
    else:
        E.info("reading from stdin")
        infile = sys.stdin

    if options.filename_map:
        E.info( "reading map" )
        map_peptide2cds = IOTools.readMap( IOTools.openFile( options.filename_map, "r" ) )
        E.info( "read map for %i identifiers" % len(map_peptide2cds) )
    else:
        map_peptide2cds = {}

    E.info( "reading cds sequences" )
        
    cds_sequences = Genomics.ReadPeptideSequences( IOTools.openFile(options.filename_cdna, "r") )    

    E.info( "read %i cds sequences" % len(cds_sequences))

    ninput, noutput = 0, 0
    nskipped, nnosequence = 0, 0

    # iterate over peptide sequences
    iterator = FastaIterator.FastaIterator( infile )

    use_cds_id = options.output_identifier == "cds"

    for cur_record in iterator:

        ninput += 1
        
        peptide_identifier = re.split("\s+", cur_record.title)[0]
        cds_identifier = map_peptide2cds.get( peptide_identifier, peptide_identifier )

        if cds_identifier not in cds_sequences:
            nnosequence += 1
            continue

        p = cur_record.sequence
        c = cds_sequences[cds_identifier]
        
        E.debug("processing %s: laa=%i (without gaps=%i), lna=%i" % (peptide_identifier, len(p), len(re.sub("-", "", p)), len(c)))

        try:
            map_p2c = getMapPeptide2Cds( p, c, options )
        except ValueError:
            nskipped += 1
            continue
            
        if use_cds_id:
            identifier = cds_identifier
        else:
            identifier = peptide_identifier

        if options.output_format =="alignment":
            options.stdout.write("\t".join( map(str, (identifier, alignlib.AlignmentFormatEmissions( map_p2c ),
                                                      len(cur_record.sequence), len(cds_sequences[identifier])) ) )+"\n")
            
        elif options.output_format == "fasta":

            map_p2c.switchRowCol()

            alignatum = alignlib.makeAlignatum( c )
            
            alignatum.mapOnAlignment( map_p2c, len(p) * 3 )

            s = alignatum.getString()
            if len(s) != len(p) * 3:
                raise ValueError ("incomplete aligned string for %s: %s, cds=%s" % (cur_record.title, s, c ))
            
            options.stdout.write( ">%s\n%s\n" % (identifier, s ))

        noutput += 1
        sys.stdout.flush()

    E.info( "ninput=%i, noutput=%i, nnosequence=%i, nskipped=%i" % (ninput, noutput, nnosequence, nskipped) )
        
    E.Stop()
            




            



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
graph_combine_links_redundant.py - remove redundant links
=========================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

remove redundant links from a sorted graph.

Usage
-----

Example::

   python graph_combine_links_redundant.py --help

Type::

   python graph_combine_links_redundant.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import getopt

import CGAT.Experiment as E

param_long_options=["verbose=", "help", "version" ]
param_short_options="v:h"

param_loglevel = 1
##------------------------------------------------------------
if __name__ == '__main__':

    try:
        optlist, args = getopt.getopt(sys.argv[1:],
                                      param_short_options, param_long_options)
    except getopt.error, msg:
        print globals()["__doc__"], msg
        sys.exit(2)

    for o,a in optlist:
        if o in ( "-v", "--verbose" ):
            param_loglevel = int(a)
        elif o in ( "--version", ):
            print "version="
            sys.exit(0)
        elif o in ( "-h", "--help" ):
            print globals()["__doc__"]
            sys.exit(0)
            
    if len(args) > 0:
        print USAGE, "no arguments required."
        sys.exit(1)

    print E.GetHeader()
    print E.GetParams()

    print "token1\ttoken2\tmin\tmax\tcount\ttotal\tavg"

    last = None
    n = 0
    
    for line in sys.stdin:
        
        if line[0] == "#": continue
        
        data = line[:-1].split("\t")[:3]
        # ignore header lines
        try:
            data[2] = float(data[2])
        except ValueError:
            continue

        if not last:
            last = data
            mmin, mmax, mtotal, n  = data[2], data[2], data[2], 1
            continue
        
        if last[0] != data[0] or last[1] != data[1]:
            print string.join( map( str, (last[0], last[1], mmin, mmax, n, mtotal, mtotal/n)), "\t" )
            last = data
            mmin, mmax, mtotal, n  = data[2], data[2], 0, 0
            
        mtotal += data[2]
        mmin = min(mmin, data[2] )
        mmax = max(mmax, data[2] )
        n += 1

    if last:
        print string.join( map(str, (last[0], last[1], mmin, mmax, n, mtotal, mtotal/n)), "\t" )        
            
            
    print E.GetFooter()        



####
####
##
##
## Copyright (C) 2008 Andreas Heger All rights reserved
##
## Author: Andreas Heger <heger@ebi.ac.uk>
##
## $Id: bed2bed.py 2861 2010-02-23 17:36:32Z andreas $
##
##
####
####
'''
vcf2vcf.py - manipulate vcf files
=================================

Purpose
-------

Manipulate vcf-formatted files.


Usage
-----

Example::

   cat in.vcf | python vcf2vcf.py - --reorder alphabetical > sorted.vcf

This command generates a sorted vcf with the sample columns in in.vcf in alphabetical order.

Type::

   python vcf2vcf.py --help

for command line usage.

Methods
-------

This script provides the following methods:

re-order
   reorder sample columns in vcf formatted file according to a given sort order

Documentation
-------------

This is a tool for manipulating vcf-formatted files.  The following options are available:

+-----------+-------------------------+
|--reorder  |reorders sample columns  |
+-----------+-------------------------+

Reorder
^^^^^^^

To sort sample columns into alphabetical order::

   cat example.vcf | python vcf2vcf.py - --reorder alphabetical

This will sort the columns in the example.vcf into the order "#CHROM, POS, ID, REF, ALT, QUAL, FILTER, INFO, FORMAT,
SAMPLE_A, SAMPLE_B, SAMPLE_C, ..."

To specify a non-alphabetical order::

   cat example.vcf | python vcf2vcf.py - --reorder SAMPLE_C,SAMPLE_A,SAMPLE_B,...

This will sort the columns in the example.vcf into the order "#CHROM, POS, ID, REF, ALT, QUAL, FILTER, INFO, FORMAT,  |
SAMPLE_C, SAMPLE_A, SAMPLE_B, ..."

Code
----
'''

import sys
import re
import string
import optparse
import time
import os
import itertools
import tempfile
import subprocess
import shutil

import CGAT.Experiment as E
import CGAT.Stats as Stats
import CGAT.VCF as VCF
    
def main( argv = sys.argv ):

    parser = E.OptionParser( version = "%prog version: $Id: bed2bed.py 2861 2010-02-23 17:36:32Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option( "--reorder", dest="reorder", type="string",
                       help="reorder columns. Give column names as comma-separated list or specify ``alphabetical`` [default=%default]"  )

    parser.set_defaults( reorder = None )
    
    (options, args) = E.Start( parser, add_pipe_options = True )


    noutput = 0
    
    infile = VCF.VCFFile( open(args[0], "r") )

    if options.reorder: 
        order = options.reorder.split(",")
        if "alphabetical" in order:
            order = sorted(infile.samples)
    else:
        order = False

    infile.writeHeader( options.stdout, order = order )
        
    for vcf in infile:
        if order: vcf.order = order 
        options.stdout.write( str(vcf) + "\n" )

    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv ) )



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
psl2stats.py - output genomic coverage from psl formatted alignments
====================================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Example::

   python psl2stats.py --help

Type::

   python psl2stats.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import tempfile
import subprocess
import optparse
import math


import CGAT.Experiment as E
import CGAT.Blat as Blat
import CGAT.IOTools as IOTools

# import psyco_full
import sys
import bx.bitset
import bx.bitset_builders

##---------------------------------------------------------------------------------------------
    
if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: psl2stats.py 2781 2009-09-10 11:33:14Z andreas $",
                                    usage = globals()["__doc__"])

    parser.set_defaults(
        )
    
    (options, args) = E.Start( parser )

    query_bitsets, target_bitsets = {}, {}

    def addRange( bitset, id, size, iterator ):
        
        if id not in bitset: bitset[id] = bx.bitset.BinnedBitSet( size )
        b = bitset[id]

        for start, end in iterator:
            b.set_range( start, end-start )

    for psl in Blat.iterator( options.stdin ):

        addRange( query_bitsets, 
                  psl.mQueryId, 
                  psl.mQueryLength,
                  psl.iterator_query_exons() )

        addRange( target_bitsets, 
                  psl.mSbjctId, 
                  psl.mSbjctLength,
                  psl.iterator_sbjct_exons() )
        
    def printBitset( outfile, bitsets ):


        outfile.write( "contig\tcovered\tsize\tpcovered\n" )
        total, total_len = 0, 0
        for chrom in sorted(bitsets):
            
            l = bitsets[chrom].size 
            s = bitsets[chrom].count_range( 0, l )
            if l > 0:
                outfile.write( "%s\t%i\t%i\t%6.4f\n" % (chrom, s,l,100.0 * s / l) )
            total += s
            total_len += l

        if total_len > 0:
            outfile.write("total\t%i\t%i\t%6.4f\n" % (total,total_len, 100.0 * total / total_len))        
        
    options.stdout.write("# query\n" )
    printBitset( options.stdout, query_bitsets )
    options.stdout.write("# target\n" )
    printBitset( options.stdout, target_bitsets )

    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
bam2stats.py - compute stats from a bam-file
============================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script takes a bam file as input and computes a few metrics:


+------------------------+------------------------------------------+
|*Category*              |*Content*                                 |
+------------------------+------------------------------------------+
|total                   |total number of alignments in bam file    |
+------------------------+------------------------------------------+
|alignments_mapped       |alignments mapped to a chromosome (bam    |
|                        |flag)                                     |
+------------------------+------------------------------------------+
|alignments_unmapped     |alignments unmapped (bam flag)            |
+------------------------+------------------------------------------+
|qc_fail                 |alignments failing QC (bam flag)          |
+------------------------+------------------------------------------+
|mate_unmapped           |alignments in which the mate is unmapped  |
|                        |(bam flag)                                |
+------------------------+------------------------------------------+
|reverse                 |alignments in which read maps to reverse  |
|                        |strand (bam flag)                         |
+------------------------+------------------------------------------+
|mate_reverse            |alignments in which mate maps to reverse  |
|                        |strand (bam flag)                         |
+------------------------+------------------------------------------+
|proper_pair             |alignments in which both pairs have been  |
|                        |mapped properly ly (according to the      |
|                        |mapper) (bam flag)                        |
+------------------------+------------------------------------------+
|read1                   |alignments for 1st read of pair (bam flag)|
+------------------------+------------------------------------------+
|paired                  |alignments of reads that are paired (bam  |
|                        |flag)                                     |
+------------------------+------------------------------------------+
|duplicate               |read is PCR or optical duplicate (bam     |
|                        |flag)                                     |
+------------------------+------------------------------------------+
|read2                   |alignment is for 2nd read of pair (bam    |
|                        |flag)                                     |
+------------------------+------------------------------------------+
|secondary               |alignment is not primary alignment        |
+------------------------+------------------------------------------+
|alignments_rna          |          alignments mapping to regions of|
|                        |repetitive RNA                            |
+------------------------+------------------------------------------+
|alignments_no_rna       |alignments mapping not to regions of      |
|                        |repetitive RNA (if --remove -rna has been |
|                        |set, otherwise equal to mapped)           |
+------------------------+------------------------------------------+
|alignments_duplicates   |number of alignments mapping to the same  |
|                        |location                                  |
+------------------------+------------------------------------------+
|alignments_unique       |number of alignments mapping to unique    |
|                        |locations                                 |
+------------------------+------------------------------------------+
|reads_total             |number of reads in file. Either given via |
|                        |--input-reads or deduc ed as the sum of   |
|                        |mappend and unmapped reads                |
+------------------------+------------------------------------------+
|reads_mapped            |number of reads mapping in file. Derived  |
|                        |from the total number o f alignments and  |
|                        |removing counts for multiple              |
|                        |matches. Requires the NH flag to be set   |
|                        |correctly.                                |
+------------------------+------------------------------------------+
|reads_unmapped          |number of reads unmapped in file. Assumes |
|                        |that there is only one                    |
|                        | entry per unmapped read.                 |
+------------------------+------------------------------------------+
|reads_missing           |number of reads missing, if number of     |
|                        |reads given by --input-rea ds. Otherwise  |
|                        |0.                                        |
+------------------------+------------------------------------------+
|reads_norna             |reads not mapping to repetetive RNA       |
|                        |regions.                                  |
+------------------------+------------------------------------------+
|pairs_total             |number of total pairs - this is the number|
|                        |of reads_total divide d by two. If there  |
|                        |were no pairs, pairs_total will be 0.     |
+------------------------+------------------------------------------+
|pairs_mapped            |number of mapped pairs - this is the same |
|                        |as the number of prope r pairs.           |
+------------------------+------------------------------------------+

Additionally, the script outputs histograms for the following tags and
scores. These histograms are only computed for alignments not within
regions of repetitive RNA.

* NM: number of mismatches in alignments.
* NH: number of hits of reads.
* mapq: mapping quality of alignments.

Usage
-----

Example::

   cat in.bam | python bam2stats.py -

This command will generate various statistics based on the supplied BAM file, such as
percentage reads mapped and percentage reads mapped in pairs.

Type::

   python cgat_script_template.py --help

for command line help.

Documentation
-------------

Reads are not counted via read name, but making use of NH and HI flags
when present.  To recap, NH is the number of reported alignments that
contain the query in the current record, while HI is the hit index and
ranges from 0 to NH-1.

Unfortunately, not all aligners follow this convention. For example,
gsnap seems to set NH to the number of reportable alignments, while
the actual number of reported alignments in the file is less. Thus, if
the HI flag is present, the maximum HI is used to correct the NH
flag. The assumption is, that the same reporting threshold has been
used for all alignments.

If no NH flag is present, it is assumed that all reads have only been reported once.

Multi-matching counts after filtering are really guesswork. Basically,
the assumption is that filtering is consistent and will tend to remove
all alignments of a query.

Code
----

'''

import os
import sys
import re
import optparse
import collections
import CGAT.Experiment as E
import CGAT.IOTools as IOTools
import CGAT.GTF as GTF
import pysam

try:
    import pyximport
    pyximport.install(build_in_temp=False)
    import _bam2stats
except ImportError:
    import CGAT._bam2stats as _bam2stats

FLAGS = {
    1: 'paired',
    2: 'proper_pair',
    4: 'unmapped',
    8: 'mate_unmapped',
    16: 'reverse',
    32: 'mate_reverse',
    64: 'read1',
    128: 'read2',
    256: 'secondary',
    512: 'qc_fail',
    1024: 'duplicate'  }

def computeMappedReadsFromAlignments( total_alignments, nh, max_hi ):
    '''compute number of reads alignment from total number of alignments'''
    nreads_mapped = total_alignments
    if len(nh) > 0:
        max_nh = max(nh.keys())
        if max_hi > 0:
            for x in xrange( 2, min( max_nh + 1, max_hi )): 
                nreads_mapped -= (nh[x] / x) * (x-1)
            for x in xrange( max_hi, max_nh + 1): 
                nreads_mapped -= (nh[x] / max_hi) * (max_hi-1)
        else:
            for x in xrange( 2, max(nh.keys() ) + 1 ): nreads_mapped -= (nh[x] / x) * (x-1)

    return nreads_mapped

def writeNH( outfile, nh, max_hi ):
    '''output nh array, correcting for max_hi if less than nh'''

    # need to remove double counting
    # one read matching to 2 positions is only 2

    max_nh = max(nh.keys())
    if max_hi > 0:
        for x in xrange( 1, min( max_nh + 1, max_hi) ): 
            if nh[x] == 0: continue
            outfile.write("%i\t%i\n" % (x, nh[x] / x))
        for x in xrange( max_hi, max_nh + 1): 
            if nh[x] == 0: continue
            outfile.write("%i\t%i\n" % (x, nh[x] / max_hi))
    else:
        for x in xrange( 1, max_nh + 1):
            if nh[x] == 0: continue
            outfile.write("%i\t%i\n" % (x, nh[x] / x))


def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option( "-r", "--filename-rna", dest="filename_rna", type="string", metavar='GFF',
                       help = "gff formatted file with rna locations. Note that the computation currently "
                              "does not take into account indels, so it is an approximate count only [%default]" )
    parser.add_option( "-f", "--remove-rna", dest="remove_rna", action="store_true",
                       help = "as well as counting, also remove rna reads for duplicate and other counts [%default]" )
    parser.add_option( "-i", "--input-reads", dest="input_reads", type="int",
                       help = "the number of reads - if given, used to provide percentages [%default]" )
    parser.add_option( "--force-output", dest="force_output", action="store_true",
                       help = "output nh/nm stats even if there is only a single count [%default]" )

#    parser.add_option( "-p", "--ignore-pairs", dest="ignore_pairs", action="store_true",
#                       help = "if set, pairs will be counted individually. The default is to count a pair as one [%default]" )
                       
    parser.set_defaults(
        filename_rna = None,
        remove_rna = False,
        ignore_pairs = False,
        input_reads = 0,
        force_output = False,
        )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv, add_output_options = True )

    if options.filename_rna:
        rna = GTF.readAndIndex( GTF.iterator( IOTools.openFile( options.filename_rna ) ) )
    else:
        rna = None

    pysam_in = pysam.Samfile( "-", "rb" )

    c, flags_counts, nh_filtered, nh_all, nm_filtered, nm_all, mapq, mapq_all, max_hi = \
        _bam2stats.count( pysam_in, options.remove_rna, rna )

    if max_hi > 0 and max_hi != max( nh_all.keys() ):
        E.warn( "max_hi(%i) is inconsistent with max_nh (%i) - counts will be corrected" \
                    % (max_hi, max(nh_all.keys())))
    flags = sorted(flags_counts.keys())

    outs = options.stdout
    outs.write( "category\tcounts\tpercent\tof\n" )
    outs.write( "alignments_total\t%i\t%5.2f\ttotal\n" % (c.input, 100.0 ) )
    if c.input == 0: 
        E.warn( "no input - skipped" )
        E.Stop()
        return

    nalignments_unmapped = flags_counts["unmapped"]
    nalignments_mapped = c.input - nalignments_unmapped
    outs.write( "alignments_mapped\t%i\t%5.2f\ttotal\n" % \
                    (nalignments_mapped, 100.0 * nalignments_mapped / c.input ) )
    outs.write( "alignments_unmapped\t%i\t%5.2f\ttotal\n" % \
                    ( nalignments_unmapped, 100.0 * nalignments_unmapped / c.input ) )

    if nalignments_mapped == 0: 
        E.warn( "no alignments - skipped" )
        E.Stop()
        return

    for flag, counts in flags_counts.iteritems():
        if flag == "unmapped": continue
        outs.write( "%s\t%i\t%5.2f\talignments_mapped\n" % ( flag, counts, 100.0 * counts / nalignments_mapped ) )

    if options.filename_rna:
        outs.write( "alignments_rna\t%i\t%5.2f\talignments_mapped\n" % (c.rna, 100.0 * c.rna / nalignments_mapped ) )
        outs.write( "alignments_no_rna\t%i\t%5.2f\talignments_mapped\n" % (c.filtered, 100.0 * c.filtered / nalignments_mapped ) )
        normby = "norna"
    else:
        normby = "mapped"
    
    if c.filtered > 0:
        outs.write( "alignments_duplicates\t%i\t%5.2f\t%s\n" % (c.duplicates, 100.0* c.duplicates / c.filtered, normby))
        outs.write( "alignments_unique\t%i\t%5.2f\t%s\n" % (c.filtered - c.duplicates,
                                                            100.0*(c.filtered - c.duplicates)/c.filtered,
                                                            normby) )

    # derive the number of mapped reads in file from alignment counts
    nreads_unmapped = flags_counts["unmapped"]
    nreads_mapped = computeMappedReadsFromAlignments( nalignments_mapped, nh_all, max_hi )
    
    nreads_missing = 0
    if options.input_reads:
        nreads_total = options.input_reads
        # unmapped reads in bam file?
        if nreads_unmapped: 
            nreads_missing = nreads_total - nreads_unmapped - nreads_mapped
        else: 
            nreads_unmapped = nreads_total - nreads_mapped

    elif nreads_unmapped:
        # if unmapped reads are in bam file, take those
        nreads_total = nreads_mapped + nreads_unmapped
    else:
        # otherwise normalize by mapped reads
        nreads_unmapped = 0
        nreads_total = nreads_mapped

    outs.write( "reads_total\t%i\t%5.2f\treads_total\n" % (nreads_total, 100.0 ) )
    outs.write( "reads_mapped\t%i\t%5.2f\treads_total\n" % (nreads_mapped, 100.0 * nreads_mapped / nreads_total ) )
    outs.write( "reads_unmapped\t%i\t%5.2f\treads_total\n" % (nreads_unmapped, 100.0 * nreads_unmapped / nreads_total ) )
    outs.write( "reads_missing\t%i\t%5.2f\treads_total\n" % (nreads_missing, 100.0 * nreads_missing / nreads_total ) )

    if len(nh_all) > 1:
        outs.write( "reads_unique\t%i\t%5.2f\treads_mapped\n" % (nh_all[1], 100.0 * nh_all[1] / nreads_mapped ) )

    # compute after filtering
    # not that these are rough guesses
    if options.filename_rna:
        nreads_norna = computeMappedReadsFromAlignments( c.filtered, nh_filtered, max_hi )

        outs.write( "reads_norna\t%i\t%5.2f\treads_mapped\n" % (nreads_norna, 100.0 * nreads_norna / nreads_mapped ) )

        if len(nh_filtered) > 1:
            outs.write( "reads_norna_unique\t%i\t%5.2f\treads_norna\n" % (nh_filtered[1], 100.0 * nh_filtered[1] / nreads_norna ) )

    pysam_in.close()

    # output paired end data
    if flags_counts["read2"] > 0:
        pairs_total = nreads_total // 2
        pairs_mapped = flags_counts["proper_pair"] // 2
        outs.write( "pairs_total\t%i\t%5.2f\tpairs_total\n" % (pairs_total, 100.0))
        outs.write( "pairs_mapped\t%i\t%5.2f\tpairs_total\n" % (pairs_mapped, 100.0 * pairs_mapped / pairs_total))
    else:
        pairs_total = pairs_mapped = 0
        outs.write( "pairs_total\t%i\t%5.2f\tpairs_total\n" % (pairs_total,0.0))
        outs.write( "pairs_mapped\t%i\t%5.2f\tpairs_total\n" % (pairs_mapped, 0.0))

    if options.force_output or len(nm_filtered) > 0:
        outfile = E.openOutputFile( "nm", "w" )
        outfile.write( "NM\talignments\n" )
        if len(nm_filtered) > 0:
            for x in xrange( 0, max( nm_filtered.keys() ) + 1 ): outfile.write("%i\t%i\n" % (x, nm_filtered[x]))
        else:
            outfile.write( "0\t%i\n" % (c.filtered) )
        outfile.close()

    if options.force_output or len(nh_all) > 1:
        outfile = E.openOutputFile( "nh_all", "w")
        outfile.write( "NH\treads\n" )
        if len(nh_all) > 0:
            writeNH( outfile, nh_all, max_hi )
        else:
            # assume all are unique if NH flag not set
            outfile.write( "1\t%i\n" % (c.mapped_reads) )
        outfile.close()

    if options.force_output or len(nh_filtered) > 1:
        outfile = E.openOutputFile( "nh", "w")
        outfile.write( "NH\treads\n" )
        if len(nh_filtered) > 0:
            writeNH( outfile, nh_filtered, max_hi )
        else:
            # assume all are unique if NH flag not set
            outfile.write( "1\t%i\n" % (c.filtered) )
        outfile.close()

    if options.force_output or len(mapq_all) > 1:
        outfile = E.openOutputFile( "mapq", "w")
        outfile.write( "mapq\tall_reads\tfiltered_reads\n" )
        for x in xrange( 0, max( mapq_all.keys() ) + 1 ):       
            outfile.write("%i\t%i\t%i\n" % (x, mapq_all[x], mapq[x]))
        outfile.close()
        

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )

    



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: index2gff.py 2880 2010-04-07 08:44:13Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
"""
index2gff.py - convert indexed fasta file to gff file
=====================================================

:Author: Andreas Heger
:Release: $Id: index2gff.py 2880 2010-04-07 08:44:13Z andreas $
:Date: |today|
:Tags: Python

Purpose
-------

Usage
-----

Type::

   python <script_name>.py --help

for command line help.

Code
----

""" 

import os
import sys
import re
import optparse

import CGAT.IndexedFasta as IndexedFasta
import CGAT.Experiment as E

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: index2gff.py 2880 2010-04-07 08:44:13Z andreas $",
                                    usage = globals()["__doc__"] )

    parser.add_option("-g", "--genome-file", dest="genome_file", type="string",
                      help="filename with genome [default=%default]."  )

    parser.set_defaults(
        genome_file = None,
        )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    ## do sth
    ninput, nskipped, noutput = 0, 0, 0

    fasta = IndexedFasta.IndexedFasta( options.genome_file )

    entry = GTF.Entry()
    entry.start = 0
    entry.feature = "contig"
    entry.source = "genome"

    for contig, size in fasta.getContigSizes( with_synonyms = False ).iteritems():
        ninput += 1
        entry.contig = contig
        entry.end = int(size)
        options.stdout.write( "%s\n" % str(entry) )
        noutput += 1

    E.info( "ninput=%i, noutput=%i, nskipped=%i" % (ninput, noutput,nskipped) )

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )




################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
MaliIO.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Code
----

'''
import string, sys, re
try: import alignlib
except ImportError: pass

##---------------------------------------------------------------------------------------
def writeFasta( outfile,
                mali,
                line_width = 60,
                identifiers = None,
                skip_first = 0,
                gap_char = None
                ):
    
    """print multiple alignment in fasta format.
    """
    
    mali_width = mali.getWidth()
    mali_length = mali.getLength()
    
    for x in range(skip_first, mali_width):
        if identifiers:
            outfile.write(">" + str(identifiers[x]) + "\n")
        else:
            outfile.write(">%i\n" % x)
        if not gap_char:
            outfile.write(mali.getRow(x).getString() + "\n" )
        else:
            outfile.write(string.replace(mali.getRow(x).getString(), "-", gap_char) + "\n" )            

##---------------------------------------------------------------------------------------
def WriteMODELLER( outfile,
                    mali,
                    line_width = 60,
                    identifiers = None,
                    skip_first = 0,
                    gap_char = None
                    ):
    """print multiple alignment in PIR (MODELLER) format.
example:

>P1;5fd1
structureX:5fd1:1    : :106  : :ferredoxin:Azotobacter vinelandii: 1.90: 0.19
AFVVTDNCIKCKYTDCVEVCPVDCFYEGPNFLVIHPDECIDCALCEPECPAQAIFSEDEVPEDMQEFIQLNAELA
EVWPNITEKKDPLPDAEDWDGVKGKLQHLER*
    """
    
    mali_width = mali.getWidth()
    mali_length = mali.getLength()
    
    for x in range(skip_first, mali_width):
        
        if identifiers:
            outfile.write(">" + str(identifiers[x]) + "\n")
        else:
            outfile.write(">%i\n" % x)
            
        if not gap_char:
            outfile.write(mali.getRow(x).getString() + "\n" )
        else:
            outfile.write(string.replace(mali.getRow(x).getString(), "-", gap_char) + "\n" )            
    
              
##---------------------------------------------------------------------------------------
def writeClustalW( outfile, mali, line_width = 60, identifiers = None ):
    """print alignment in ClustalW format.
    (have to still look up the format, dsc can read it.)
    """

    mali_width = mali.getWidth()
    mali_length = mali.getLength()

    lines = []
    for x in range(0, mali_width):
        lines.append(mali.getRow(x).getString()) 

    outfile.write( "CLUSTAL W(1.60) multiple sequence alignment\n" )
    outfile.write( "\n" * 5 )

    for i in range(0, mali_length, line_width):
        i_to = i + line_width

        for x in range( 0, mali_width ):
            if identifiers:
                outfile.write("%-16s" % identifiers[x])
            else:
                outfile.write("%-16i" % x)

            outfile.write(lines[x][i:i_to] + "\n")
            
        outfile.write("\n" * 2)

##---------------------------------------------------------------------------------------
def readPicasso( infile ):
    """read alignment in the non-defined picasso format.
    """

    mali = alignlib.makeMultipleAlignment()

    while 1:
        line = infile.readline()
        if not line: break

        x = re.search( "\d+\s+([A-Z\-\.]*)\s+\d+", line)
        if x:
            s = x.groups()[0]
            a = alignlib.makeAlignatumFromString(s)
            a.thisown = 0
            mali.addAlignatum( a )
            
    return mali

##---------------------------------------------------------------------------------------
def compressAlignment( alignment, gap_character = "-", ignore_beginning = 0 ):
    """compress an alignment string.
    Lower-case characters at the beginning are ignored if so wished.
    --xxabBCDEfgHI
    becomes:
    -6+4

    This was necessary for radar output (e.g., 46497)
    """

    # subsitute all initial lower case characters with a gap
    if ignore_beginning:
        x = re.search("[A-Z]", alignment)
        if x:
            alignment = gap_character * x.start() + alignment[x.start():] 
        
    d = 0
    if alignment[0] == gap_character:
        gap = 1
    else:
        gap = 0
        
    result = ""
    
    for char in alignment:
        if char == gap_character and not gap:
            result += "+%i" % d
            d = 1
            gap = 1
            continue

        if char != gap_character and gap:
            result += "-%i" % d
            d = 1
            gap = 0
            continue

        d = d + 1
        
    if gap:
        result = result + "-%i" % d
    else:
        result = result + "+%i" % d
        
    return result

##---------------------------------------------------------------------------------------
def readFasta( infile, pattern_identifier="\S+" ):
    """read alignment in fasta format.
    """

    mali = {}
    id = None
    fragments = []
    identifiers = []
    for line in infile:
        if line[0] == "#": continue
        if line[0] == ">":
            if id: mali[id] = re.sub( "\s", "", string.join( fragments, ""))
            id = re.search( "^(%s)" % pattern_identifier, line[1:-1]).group(0)
            identifiers.append(id)
            fragments = []
            continue
        fragments.append( line[:-1] )

    if id: mali[id] = re.sub( "\s", "", string.join( fragments, ""))
    return convertGaps( mali ), identifiers

##---------------------------------------------------------------------------------------
def readPlain( infile ):
    
    mali = {}
    for line in infile:
        if line[0] == "#": continue
        data = line[:-1].split("\t")
        identifiers.append( data[3] )
        mali[data[3]] = data[2]
        
    return convertGaps( mali ), identifiers

##---------------------------------------------------------------------------------------
def readMali( infile, format = "fasta" ):

    if format == "fasta": return readFasta( infile )
    elif format == "plain": return readPlain( infile )
    else:
        raise ValueError, "unknown format %s" % format

##---------------------------------------------------------------------------------------
def convertGaps( mali, old_gap=".", new_gap = "-"):
    """convert gaps characters in mali.
    """
    for id in mali.keys():
        mali[id] = string.replace( mali[id], old_gap, new_gap,  )
    return mali

##---------------------------------------------------------------------------------------
def removeGappedColumns( mali, gap_char = "-" ):
    """remove all gapped columns in mali."""

    if len(mali) == 0: return mali
    
    keys = mali.keys()
    lmali = len(mali[keys[0]])
    wmali = len(keys)

    gaps_per_column = [0] * lmali

    for m in mali.values():
        for x in range(lmali):
            if m[x] == gap_char: gaps_per_column[x] += 1

    new_mali = {}
    
    for k in keys:
        s = []
        m = mali[k]
        for x in range(lmali):
            if gaps_per_column[x] < wmali:
                s += m[x]
        new_mali[k] = string.join(s, "")
        
    return new_mali

##---------------------------------------------------------------------------------------
def getSubset( mali, identifiers, not_in_set = False ):
    """return subset of mali which only contains identifiers."""

    new_mali = {}

    if not_in_set:
        for k,v in mali.items():
            if k not in identifiers: new_mali[k] = v
    else:
        for k,v in mali.items():
            if k in identifiers: new_mali[k] = v
        
        
    return new_mali

##---------------------------------------------------------------------------------------
def getFrameColumnsForMaster( mali, master, gap_char = "-" ):
    """get columns in frame according to master.
    """
    columns = []
    sequence = mali[master]
    x = 0
    while x < len(sequence):
        c = 0
        codon = []
        while x < len(sequence) and c < 3:
            if  sequence[x] != gap_char: #  and sequence[x] in string.uppercase:
                codon.append( x )
                c += 1
            x += 1
            
        if len(codon) == 3: columns.append( codon )

    return columns

##---------------------------------------------------------------------------------------
def getFrameColumnsForMasterPattern( mali, identifiers, master_pattern, gap_char = "-"):
    """get columns in frame for all masters matching the pattern.
    """
    columns = []
    for id in identifiers:
        if re.search( master_pattern, id ):
            columns += getFrameColumnsForMaster( mali, id )

    if len(columns) == 0:
        columns += getFrameColumnsForMaster( mali, identifiers[0] )

    # sort all columns by tuple. The "shortest" codon will be first (1,2,3) before (1,2,100)
    columns.sort()

    # select codons
    frame_columns = []
    last_codon = columns[0]
    for codon in columns[1:]:
        # skip identical codons
        if codon == last_codon: continue

        # take first (shortest) codon in case of identical first residue
        if codon[0] == last_codon[0]: continue

        # if not overlapping, keep
        if codon[0] > last_codon[2]:
            frame_columns.append( last_codon )

        # if overlapping, but out of register: skip
        last_codon = codon

    frame_columns.append( last_codon )
    
    return frame_columns

##---------------------------------------------------------------------------------------
def getMapFromMali( seq1, seq2, gap_char = "-" ):
    """build map of positions between mali."""
    xpos = 0
    ypos = 0

    map_a2b = alignlib.makeAlignataVector()
    # build map between genomic sequences:
    for p in range(len(seq1)):

        if     seq1[p] != gap_char and \
               seq2[p] != gap_char and \
               seq1[p] in string.uppercase and \
               seq2[p] in string.uppercase:
            map_a2b.addPairExplicit( xpos + 1, ypos + 1, 0)
            
        if seq1[p] != gap_char:
            xpos += 1
        if seq2[p] != gap_char:
            ypos += 1
    return map_a2b


##---------------------------------------------------------------------------------------
def getCodonSequence( sequence, frame_columns, gap_char = "-", remove_stops = True ):
    """return a pruned sequence given frame columns.

    everything not in frame is deleted, only complete codons are kept.
    """
    
    fragments = []
    nstops, ncodons, naligned = 0, 0, 0
    for a,b,c in frame_columns:
        codon = sequence[a] + sequence[b] + sequence[c]

        codon_is_aligned = False
        codon_is_ok = True
        for x in codon:
            ## a codon will be masked, if it either
            ## 1. contains a gap character
            ## 2. is an unaligned character (lowercase)
            residue_is_unaligned = (x == gap_char) or \
                                   (x in string.lowercase)
            codon_is_aligned = codon_is_aligned or not residue_is_unaligned
            codon_is_ok = codon_is_ok and not residue_is_unaligned

        if codon_is_aligned: naligned += 1

        if codon_is_ok:
            ncodons += 1
            if string.upper(codon) in ("TAG", "TAA", "TGA"):
                if remove_stops:
                    fragments.append( gap_char * 3 )
                else:
                    fragments.append( codon )                        
                nstops += 1
            else:
                fragments.append( codon )                                            
        else:
            fragments.append( gap_char * 3 )

    return string.join(fragments, ""), naligned, ncodons, nstops

##---------------------------------------------------------------------------------------
def getPercentIdentity( seq1, seq2, gap_char = "-" ):
    """get number of identical residues between seq1 and seq2."""

    ntotal = 0
    nidentical = 0
    for a in range(len(seq1)):
        if seq1[a] != gap_char and seq2[a] != gap_char:
            ntotal += 1
            if seq1[a] == seq2[a]:
                nidentical += 1

    if ntotal == 0: return 0.0
    return float(nidentical) / ntotal
            
    
if __name__ == '__main__':

    c = pairsdblib.ConnectionPicasso()
    c.Connect()

    import pairsdblib
    mali = pairsdblib.makeMultipleAlignmentNeighbours( c, 54, "pairsdb_90x90")
    
    writeClustalW( sys.stdout, mali )

    c.Disconnect()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
rnaseq_bams2bam.py - merge genomic and transcriptome mapped bamfiles
====================================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script takes as input two BAM files from an RNASeq experiment. 
The first bam file (:file:`bamG`) should contain reads mapped against the genome
using a mapper permitting splicing (e.g. tophat). The second bam file
(:file:`bamT`) should contain reads mapped against known transcripts. This script
will write a new bam file that removes reads from :term:`bamG` that 
map to regions that are conflicting with those in :term:`bamT`.

.. note:: 
   Note that if junctions are supplied, the resultant bam files will not
   be sorted by position.

.. glossary::

   bamG
      :term:`bam` formatted file with reads mapped against the genome

   bamT
      :term:`bam` formatted file with reads mapped against transcripts

Usage
-----

Example::

   python cgat_script_template.py bamT.bam bamG.bam

Type::

   python cgat_script_template.py --help

for command line help.

Documentation
-------------

The script needs to look-up reads via their names. It thus builds
an index of reads mapping 

This script requires the NM attributes to be set. If it is not 
set, you will need to set a policy.

Code
----

'''

import os
import sys
import re
import optparse
import time

import CGAT.Experiment as E
import CGAT.GTF as GTF
import CGAT.IOTools as IOTools
import CGAT.Bed as Bed
import pysam
import CGAT.IndexedGenome as IndexedGenome

try:
    import pyximport
    pyximport.install(build_in_temp=False)
    import _rnaseq_bams2bam
except ImportError:
    import CGAT._rnaseq_bams2bam as _rnaseq_bams2bam

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option( "-g", "--filename-gtf", dest="filename_gtf", type="string",
                       help = "filename with gene models in gtf format [%default]" )

    parser.add_option( "-m", "--filename-mismapped", dest="filename_mismapped", type="string",
                       help = "output bam file for mismapped reads [%default]" )

    parser.add_option( "-j", "--filename-junctions", dest="filename_junctions", type="string",
                       help = "bam file with reads mapped across junctions [%default]" )

    parser.add_option( "-r", "--filename-regions", dest="filename_regions", type="string",
                       help = "filename with regions to remove in bed format [%default]" )

    parser.add_option( "-t", "--filename-transcriptome", dest="filename_transcriptome", type="string",
                       help = "bam file with reads mapped against transcripts [%default]" )

    parser.add_option( "-p", "--filename-map", dest="filename_map", type="string",
                       help = "filename mapping transcript numbers (used by --filename-transciptome) to transcript names (used by --filename-gtf) [%default]" )

    parser.add_option( "-s", "--filename-stats", dest="filename_stats", type="string",
                       help = "filename to output stats to [%default]" )

    parser.add_option( "-o", "--colour", dest="colour_mismatches", action="store_true",
                       help = "mismatches will use colour differences (CM tag) [%default]" )

    parser.add_option( "-i", "--ignore-mismatches", dest="ignore_mismatches", action="store_true",
                       help = "ignore mismatches [%default]" )

    parser.add_option( "-c", "--remove-contigs", dest="remove_contigs", type="string",
                       help = "','-separated list of contigs to remove [%default]" )

    parser.add_option( "-f", "--force", dest="force", action = "store_true",
                       help = "force overwriting of existing files [%default]" )

    parser.add_option( "-u", "--unique", dest="unique", action = "store_true",
                       help = "remove reads not matching uniquely [%default]" )

    parser.add_option( "--sam", dest="output_sam", action="store_true",
                       help = "output in sam format [%default]" )

    parser.set_defaults(
        filename_gtf = None,
        filename_mismapped = None,
        filename_junctions = None,
        filename_transcriptome = None,
        filename_map = None,
        remove_contigs = None,
        force = False,
        unique = False,
        colour_mismatches = False,
        ignore_mismatches = False,
        output_sam = False,
        filename_table = None,
        )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    if len(args) != 1:
        raise ValueError( "please supply one bam file" )

    bamfile_genome = args[0]
    genome_samfile = pysam.Samfile( bamfile_genome, "rb" )

    if options.remove_contigs:
        options.remove_contigs = options.remove_contigs.split(",")

    if options.filename_map:
        E.info( "reading map" )
        id_map = IOTools.readMap( IOTools.openFile( options.filename_map), has_header = True )
	id_map = dict( [ (y,x) for x,y in id_map.iteritems() ])
    else:
        id_map = None
        
    transcripts = {}
    if options.filename_gtf:
        E.info( "indexing geneset" )
        mapped, missed = 0, 0
        for gtf in GTF.transcript_iterator( GTF.iterator( IOTools.openFile(options.filename_gtf) )):
            gtf.sort( key = lambda x: x.start )
            transcript_id = gtf[0].transcript_id
            if id_map:
                try:
                    transcript_id = id_map[transcript_id]
                    mapped += 1
                except KeyError:
                    missed += 1
                    continue
            transcripts[transcript_id] = gtf

        E.info( "read %i transcripts from geneset (%i mapped, %i missed)" % (len(transcripts), mapped, missed) )

    regions_to_remove = None
    if options.filename_regions:
        E.info( "indexing regions" )
        regions_to_remove = IndexedGenome.Simple()
        for bed in Bed.iterator( IOTools.openFile( options.filename_regions )):
            regions_to_remove.add( bed.contig, bed.start, bed.end )
        E.info( "read %i regions" % len(regions_to_remove) )

    if options.filename_transcriptome:
        transcripts_samfile = pysam.Samfile( options.filename_transcriptome, 
                                             "rb" )
    else:
        transcripts_samfile = None

    if not options.force and os.path.exists( bamfile_output ):
        raise IOError( "output file %s already exists" % bamfile_output )

    if options.output_sam:
        output_samfile = pysam.Samfile( "-", "wh", template = genome_samfile )
    else:
        output_samfile = pysam.Samfile( "-", "wb", template = genome_samfile )

    if options.filename_mismapped:
        if not options.force and os.path.exists( options.filename_mismapped ):
            raise IOError( "output file %s already exists" % options.filename_mismapped )
        output_mismapped = pysam.Samfile( options.filename_mismapped, 
                                          "wb", 
                                          template = genome_samfile )
    else:
        output_mismapped = None

    if options.filename_junctions:
        junctions_samfile = pysam.Samfile( options.filename_junctions, 
                                           "rb" )
    else:
        junctions_samfile = None

    c = _rnaseq_bams2bam.filter( genome_samfile, 
                                 output_samfile, 
                                 output_mismapped,
                                 transcripts_samfile,
                                 junctions_samfile,
                                 transcripts,
                                 regions = regions_to_remove,
                                 unique = options.unique,
                                 remove_contigs = options.remove_contigs,
                                 colour_mismatches = options.colour_mismatches,
                                 ignore_mismatches = options.ignore_mismatches,
                                 ignore_transcripts = transcripts_samfile == None,
                                 ignore_junctions = junctions_samfile == None)

    if options.filename_stats:
        outf = IOTools.openFile( options.filename_stats, "w" )
        outf.write( "category\tcounts\n%s\n" % c.asTable() )
        outf.close()

    if options.filename_transcriptome:
        transcripts_samfile.close()
    
    genome_samfile.close()
    output_samfile.close()
    if output_mismapped:
        output_mismapped.close()

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )




################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
liftover.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Example::

   python liftover.py --help

Type::

   python liftover.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import optparse
import time

"""liftover coordinates using a liftover formatted file from the ucsc
"""

import CGAT.Experiment as E
import numpy

parser = E.OptionParser( version = "%prog version: $Id: liftover.py 2404 2009-01-12 11:22:36Z andreas $")

def readLiftOver( infile, chromosome,
                  chromosome_size = 250000000,
                  report_step = 1000000 ):
    """read a matrix. There probably is a routine for this in Numpy, which
    I haven't found yet.
    """

    if options.loglevel >= 2:
        print "## started reading mapping information"
        sys.stdout.flush()
        
    map_position   = numpy.zeros( (chromosome_size,), numpy.int) 
    
    ## signed character for chromosme, negative values for negative strands
    map_chromosome = numpy.zeros( (chromosome_size,), numpy.int8 )

    map_id2chromosome = ["",]
    map_chromosome2id = {}
    n = 0
    
    for line in infile:

        n += 1

        if not (n % report_step):
            if options.loglevel >= 2:
                print "# iteration %i" % n
                sys.stdout.flush()
                
        if line[:5] == "chain":
            ( chr_x, size_x, strand_x, first_x, last_x,
              chr_y, size_y, strand_y, first_y, last_y,
              dontknow) = line[:-1].split(" ")[2:]

            if strand_x == "-":
                raise "what shall I do with negative strands?"

            x = int(first_x)
            
            ## revert coordinates for negative strands (it seems that
            ## the mapping file uses reverse coordinates, while liftover
            ## output doesn't)
            ## add 1 to coordinates, because 0 is flag for unmappable.
            if strand_y == "-":
                invert = True
                ## no +1, because already one past current residue (due to open
                ## bracket)
                y = int(size_y) - int(first_y) 
            else:
                invert = False
                y = int(first_y) + 1
            
            if chr_x != chromosome:
                keep = False
            else:
                keep = True
                if options.loglevel >= 3:
                    print "# adding alignment", line[:-1]
                    
            continue
        
        elif line.strip() == "":
            keep = False
            continue
            
        elif keep:
            
            data = map(int, line[:-1].split("\t"))
            
            if len(data) == 3:
                size, increment_x, increment_y = data
            else:
                size, increment_x, increment_y = data[0], 0, 0

            # add position
            if invert:
                map_position[ x:x+size ] = numpy.arrayrange( y, y-size, -1 )
            else:
                map_position[ x:x+size ] = numpy.arrayrange( y, y+size, 1 )

            if chr_y not in map_id2chromosome:
                map_chromosome2id[chr_y] = len(map_id2chromosome)                 
                map_id2chromosome.append( chr_y )
                
            id = map_chromosome2id[chr_y]
            if strand_y == "-": id = -id

            # add chromsome
            map_chromosome[x:x+size] = id

            x += increment_x + size
            if invert:
                y -= increment_y + size
            else:
                y += increment_y + size

            if y < 0:
                raise "illegal mapping: %i -> %i for %s %s:%s-%s(%s) to %s %s: %s-%s(%s)" % ( x, y, 
                                                                                              chr_x, strand_x, first_x, last_x, size_x,
                                                                                              chr_y, strand_y, first_y, last_y, size_y )
            
    return map_position, map_chromosome, map_chromosome2id, map_id2chromosome

if __name__ == "__main__":

    parser.add_option("-c", "--chromosome", dest="chromosome", type="string",
                      help="chromosome to take." )

    parser.add_option("-m", "--map", dest="filename_map", type="string",
                      help="filename with mapping info.",
                      metavar = "FILE")

    parser.set_defaults( \
        filename_map = "",
        chromosome = None,
        )
    
    (options, args) = E.Start( parser )

    if options.filename_map == "":
        raise "please specify the file with the liftover mapping information."

    if not options.chromosome:
        raise "please give a chromosome."        
        
    map_position, map_chromosome, map_chromosome2id, map_id2chromosome = readLiftOver( open(options.filename_map, "r"), options.chromosome )

    l = 0
    
    for line in sys.stdin:

        if line[0] == "#": continue

        data = line[:-1].split("\t")

        chromosome = data[0]
        range_from, range_to = int(data[1]), int(data[2])
        l += 1

        if chromosome == options.chromosome:

            if options.loglevel >= 1:
                print "#", l, ":", line[:-1]
            
            for x in range(range_from, range_to):
                if map_position[x]:
                    id = map_chromosome[x]
                    if id > 0:
                        c = "+"
                    else:
                        c = "-"
                        id = -id
                        
                    print "%s\t%i\t%s\t%s\t%i" % (chromosome, x, map_id2chromosome[ id ], c, map_position[x] - 1 )
                else:
                    pass
                    # print "%s\t%i\tna" % (chromosome, x )                    

    E.Stop()

    



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
"""
FastaIterator.py - iterate over fasta files
===========================================

The difference to the biopython iterator is that this one 
skips over comment lines starting with "#".

Code
----

"""
import subprocess, os

class FastaRecord:

    def __init__(self, title, sequence ):

        self.title = title
        self.sequence = sequence

def iterate( infile ):

    h = infile.readline()[:-1]

    if not h: raise StopIteration
    
    while h[0] != ">":
        h = infile.readline()[:-1]
        if not h: raise StopIteration
        continue

    h = h[1:]
    
    seq = []
    
    for line in infile:

        if line[0] == "#": continue
        
        line = line[:-1] # remove newline
        if not line: continue

        if line[0] == '>':
            yield FastaRecord(h,''.join(seq))
            
            h = line[1:]
            seq = []
            continue
        
        seq.append(line)
        
    yield FastaRecord(h,''.join(seq))

class FastaIterator:
    '''a iterator of :term:`fasta` formatted files.
    '''

    def __init__(self, f, *args, **kwargs):
        self.mIterator = iterate(f)
        
    def __iter__(self):
        return self

    def next(self):
        return self.mIterator.next()

##------------------------------------------------------------
def iterate_together( *args ):
    """iterate synchronously over one or more fasta files.

    The iteration finishes once any of the files is exhausted.

    yield output tuples of sequences."""
    iterators = [ FastaIterator( open(x, "r") ) for x in args ]

    while 1:
        yield [ x.next() for x in iterators ]


##------------------------------------------------------------
def count( filename ):
    '''count number of sequences in fasta file.'''
    if filename.endswith(".gz"):
        statement = "zcat %s | grep -c '>'" % filename
    else:
        statement = "cat %s | grep -c '>'" % filename

    if not os.path.exists( filename ):
        raise OSError( "file '%s' does not exist" % filename )

    # grep returns error if no match is found
    try:
        return subprocess.check_output( statement, shell = True )
    except subprocess.CalledProcessError:
        return 0



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
plot_matrix.py - plot a matrix (imshow)
=======================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script plots a matrix given in tabular format.

Usage
-----

Example::

   python plot_matrix.py --help

Type::

   python plot_matrix.py --help

for command line help.

Documentation
-------------

Code
----

'''
import sys
import os
import string
import re
import time
import optparse


import scipy
import numpy
import CGAT.MatlabTools as MatlabTools
import CGAT.Experiment as E
from array import array
import math
import numpy

param_grid_size_x = 6

param_hardcopy = "test.ps"
param_chunk_size = 2000
param_background = 255

def GetRange( matrix, r ):
    """get ranges from a range string.
    can be two numbers separated by comma, or min/max + a number
    """
    
    rr = r.split(",")
    
    if len(rr) != 2:
        raise "please supply two values for range separated by a comma."

    vv = []
    for x in rr:
        if x == "min": v = min(matrix.flat)
        elif x == "max": v = max(matrix.flat)
        else: v = float(x)
            
        vv.append(v)
        
    return vv

def plotMatrix( matrix, color_scheme, row_headers, col_headers, vmin, vmax, options ):

    pylab.imshow(matrix,
                 cmap=color_scheme,
                 origin='lower',
                 vmax = vmax,
                 vmin = vmin,
                 interpolation='nearest')

    # offset=0: x=center,y=center
    # offset=0.5: y=top/x=right
    offset = 0.0

    if options.xticks:
        pylab.xticks( [ offset + x for x in range(len(options.xticks)) ],
                      options.xticks,
                      rotation="vertical",
                      fontsize="8" )
    else:
        if col_headers and len(col_headers) < 100:
            pylab.xticks( [ offset + x for x in range(len(col_headers)) ],
                          col_headers,
                          rotation="vertical",
                          fontsize="8" )

    if options.yticks:
        pylab.yticks( [ offset + y for y in range(len(options.yticks)) ],
                      options.yticks,
                      fontsize="8" )
    else:
        if row_headers and len(row_headers) < 100:
            pylab.yticks( [ offset + y for y in range(len(row_headers)) ],
                          row_headers,
                          fontsize="8" )


if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: plot_matrix.py 2782 2009-09-10 11:40:29Z andreas $")
    
    parser.add_option("-c", "--columns", dest="columns", type="string",
                      help="columns to take from table." )

    parser.add_option("-a", "--hardcopy", dest="hardcopy", type="string",
                      help="write hardcopy to file.", metavar = "FILE" )

    parser.add_option("-f", "--file", dest="input_filename", type="string",
                      help="filename with table data.",
                      metavar = "FILE")

    parser.add_option("-p", "--plot", dest="plot", type="string",
                      help="plots to plot.",
                      action = "append")

    parser.add_option("-t", "--threshold", dest="threshold", type="float",
                      help="min threshold to use for counting method.")

    parser.add_option("-o", "--colours", dest="colours", type="int",
                      help="column with colour information.")

    parser.add_option("-l", "--labels", dest="labels", type="string",
                      help="column labels for x and y in matched plots.")

    parser.add_option("-e", "--headers", dest="headers", action="store_true",
                      help="headers are supplied in matrix.")

    parser.add_option( "--no-headers", dest="headers", action="store_false",
                      help="headers are not supplied in matrix.")

    parser.add_option( "--normalize", dest="normalize", action="store_true",
                      help="normalize matrix.")

    parser.add_option( "--palette", dest="palette", type="choice",
                       choices=("rainbow", "gray", "blue-white-red",
                                "autumn", "bone", "cool", "copper", "flag", "gray", "hot", "hsv", "jet", "pink", "prism",
                                "spring", "summer", "winter", "spectral",
                                "RdBu", "RdGy", "BrBG", "BuGn", "Blues", "Greens", "Reds", "Oranges", "Greys" ),
                       help="colour palette [default=%Default]")

    parser.add_option( "--reverse-palette", dest="reverse_palette", action="store_true",
                      help="reverse the palette [default=%default].")

    parser.add_option("", "--xrange", dest="xrange", type="string",
                      help="xrange."  )

    parser.add_option("", "--yrange", dest="yrange", type="string",
                      help="yrange."  )

    parser.add_option("", "--zrange", dest="zrange", type="string",
                      help="zrange."  )

    parser.add_option("", "--xticks", dest="xticks", type="string",
                      help="xticks."  )
    
    parser.add_option("", "--yticks", dest="yticks", type="string",
                      help="yticks."  )

    parser.add_option("--bar-format", dest="bar_format", type="string",
                      help="format for ticks on colourbar."  )

    parser.add_option("--title", dest="title", type="string",
                      help="title to use."  )
    
    parser.add_option("--missing", dest="missing", type="float",
                      help="value to use for missing data."  )

    parser.add_option("--subplots", dest="subplots", type="string",
                      help="split matrix into several subplots. Supply number of rows and columns separated by a comma."  )
    
    parser.set_defaults( \
        hardcopy = None,
        input_filename = "-",
        columns = "all",
        statistics = [],
        plot=[],
        threshold=0.0,
        labels = "x,y",
        colours= None,
        xrange = None,
        yrange = None,
        zrange = None,
        palette = None,
        reverse_palette = False,
        xticks = None,
        yticks = None,
        normalize = False,
        bar_format = "%1.1f",
        headers = True,
        missing = None,
        title = None,
        subplots = None)

    (options, args) = E.Start( parser )

    # import matplotlib/pylab. Has to be done here
    # for batch scripts without GUI.
    import matplotlib
    if options.hardcopy: matplotlib.use( "cairo" )
    import pylab

    if len(args) > 0:
        options.input_filename = ",".join(args)

    if options.xticks: options.xticks = options.xticks.split(",")
    if options.yticks: options.yticks = options.yticks.split(",")

    if options.xrange: options.xrange = map(float, options.xrange.split(","))
    if options.yrange: options.yrange = map(float, options.yrange.split(","))

    if options.columns != "all":
        options.columns = map( lambda x: int(x)-1, options.columns.split(","))

    filenames = options.input_filename.split(",")

    if len(filenames) > 1:
        nsubrows = (len(filenames) / 3) + 1
        nsubcols = 3
    elif options.subplots:
        nsubrows, nsubcols = [ int(x) for x in options.subplots.split( ",") ]
    else:
        nsubrows, nsubcols = 1, 1

    nsubplots = nsubrows * nsubcols

    ## Setting up color maps
    if options.palette:
        if options.palette == "gray":
            _gray_data =  {'red':   ((0., 1, 1), (1., 0, 0)),
                           'green': ((0., 1, 1), (1., 0, 0)),
                           'blue':  ((0., 1, 1), (1., 0, 0))}

            LUTSIZE = pylab.rcParams['image.lut']
            colors_gray   = matplotlib.colors.LinearSegmentedColormap('gray',   _gray_data, LUTSIZE)

    plot_id = 0
    for filename in filenames:

        plot_id += 1
        pylab.subplot( nsubrows, nsubcols, plot_id)
        
        if filename == "-":
            infile = sys.stdin
        else:
            infile = open(filename, "r")
            
        matrix,row_headers,col_headers = MatlabTools.readMatrix( infile,
                                                                 numeric_type=numpy.float32,
                                                                 take=options.columns,
                                                                 headers = options.headers,
                                                                 missing = options.missing )

        if min(matrix.flat) == max(matrix.flat):
            options.stderr.write( "matrix is uniform - no plotting done.\n")
            sys.exit(0)

        if options.normalize:
            v = max(matrix.flat)
            matrix = matrix / v
        
        if options.zrange: options.zrange = GetRange( matrix, options.zrange )

        nrows, ncols = matrix.shape
            
        if options.palette:
            if options.palette == "gray":
                color_scheme = colors_gray
            else:
                if options.reverse_palette:
                    color_scheme = eval( "pylab.cm.%s_r" % options.palette)                    
                else:
                    color_scheme = eval( "pylab.cm.%s" % options.palette)
        else:
            color_scheme = None

        if options.zrange:
            vmin, vmax = options.zrange
            for x in range(nrows):
                for y in range(ncols):
                    if matrix[x,y] < options.zrange[0]:
                        matrix[x,y] = options.zrange[0]
                    if matrix[x,y] > options.zrange[1]:
                        matrix[x,y] = options.zrange[1]
        else:
            vmin, vmax = None, None
        
        if options.subplots:

            if nsubcols > 1:
                increment_x = int(float(nrows+1) / nsubcols)
                increment_y = nrows
                
                x = 0
                y = 0
                for n in range(nsubplots):
                    pylab.subplot( nsubrows, nsubcols, plot_id)
                    plot_id +=1 

                    print n, "rows=",nsubrows, "cols=", nsubcols, y, y+ increment_y, x, x + increment_x
                    print matrix[y:y+increment_y,x:x+increment_x].shape
                    print matrix.shape
                    plotMatrix( matrix[y:y+increment_y,x:x+increment_x], 
                                color_scheme, 
                                row_headers[y:y+increment_y], 
                                col_headers[x:x+increment_x], 
                                0, 100, options )
                
                x += increment_x

            elif nsubrows > 1:
                increment_x = int(float(ncols+1) / nsubrows)
                
                x = 0
                for n in range(nsubplots):
                    pylab.subplot( nsubrows, nsubcols, plot_id)
                    plot_id +=1 
                    plotMatrix( matrix[0:nrows,x:x+increment_x], 
                                color_scheme, 
                                row_headers, 
                                col_headers[x:x+increment_x], 
                                vmin, vmax, options )
                
                    x += increment_x
        else:
            plotMatrix( matrix, color_scheme, row_headers, col_headers, vmin, vmax, options )

        if options.xrange:
            pylab.xlim( options.xrange )
            
        if options.yrange:
            pylab.ylim( options.yrange )

        if options.labels:
            xlabel, ylabel = options.labels.split(",")
            pylab.xlabel(xlabel)
            pylab.ylabel(ylabel)

        if not options.subplots:
            pylab.colorbar( format = options.bar_format)

        if options.title == None or options.title != "":
            pylab.title( filename )
        
    if options.hardcopy:
        pylab.savefig( os.path.expanduser(options.hardcopy) )
    else:
        pylab.show()

    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
malis2masks.py - mask positions in a set of malis
=================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

mask positions in a series of multiple alignments.

Output is a file with masking information.

Usage
-----

Example::

   python malis2masks.py --help

Type::

   python malis2masks.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import optparse
import math
import time
import random
import types

import CGAT.Experiment as E
import CGAT.Blat as Blat
import CGAT.Iterators as Iterators
import alignlib
import CGAT.IndexedFasta as IndexedFasta

def fillAlignment( map_alignment, alignment ):

    i = 0
    for x,c in enumerate( alignment ):
        if c != "-":
            map_alignment.addPair( i, x )
            i += 1


def main():

    parser = E.OptionParser( version = "%prog version: $Id: malis2masks.py 2781 2009-09-10 11:33:14Z andreas $", usage = globals()["__doc__"])

    parser.add_option("--random-proportion", dest="random_proportion", type="float",
                      help="mask randomly columns in multiple alignments [default=%default]" )

    parser.add_option("--random", dest="random", action="store_true",
                      help="shuffle quality scores before masking [default=%default]" )

    parser.set_defaults(
        quality_threshold = 40,
        quality_file = "quality",
        filename_map = None,
        frame = 3,
        )

    (options, args) = E.Start( parser )

    ##################################################
    ##################################################
    ##################################################
    ## read map
    ##################################################
    infile = open(options.filename_map) 
    map_genes2genome = {}
    for match in Blat.iterator( infile ):
        assert match.mQueryId not in map_genes2genome, "duplicate entry %s" % match.mQueryId
        map_genes2genome[match.mQueryId] = match
    infile.close()

    ##################################################
    ##################################################
    ##################################################
    ## get quality scores
    ##################################################
    quality = IndexedFasta.IndexedFasta( options.quality_file )
    quality.setTranslator( IndexedFasta.TranslatorBytes() )

    ##################################################
    ##################################################
    ##################################################
    ## main loop
    ##################################################
    ninput, noutput, nmissed = 0, 0, 0

    options.stdout.write( "cluster_id\tstart\tend\n" )

    for line in options.stdin:
        if line.startswith("cluster_id"): continue
        ninput += 1
        cluster_id, gene_id, alignment = line[:-1].split("\t")

        if gene_id not in map_genes2genome:
            nmissed += 1
            E.warn( "gene_id %s not found in map." % gene_id )
            continue
        
        match = map_genes2genome[gene_id]
        map_gene2genome = match.getMapQuery2Target()
        is_negative = match.strand == "-"

        # if strand is negative, the coordinates are 
        # on the negative strand of the gene/query
        # in order to work in the right coordinate system
        # revert the sequence
        if is_negative: 
            alignment = alignment[::-1]

        # get map of gene to alignment
        map_gene2mali = alignlib.makeAlignmentVector()
        fillAlignment( map_gene2mali, alignment )

        # get quality scores
        quality_scores = quality.getSequence( match.mSbjctId, "+", match.mSbjctFrom, match.mSbjctTo)


        # print str(alignlib.AlignmentFormatEmissions( map_gene2genome))
        # print str(alignlib.AlignmentFormatEmissions( map_gene2mali))
        # print quality_scores

        map_mali2genome = alignlib.makeAlignmentVector()
        alignlib.combineAlignment( map_mali2genome, map_gene2mali, map_gene2genome, alignlib.RR )
        # print str(alignlib.AlignmentFormatEmissions( map_mali2genome))

        # shuffle quality scores, but only those that are aligned
        if options.random:
            positions = []
            for fp,c in enumerate(alignment):
                if c == "-": continue
                y = map_mali2genome.mapRowToCol( fp ) - match.mSbjctFrom 
                if y < 0: continue
                positions.append( y )
            scores = [ quality_scores[ x ] for x in positions ]
            random.shuffle(scores)
            for p,q in zip( positions,scores): quality_scores[p] = q

        # negative strand
        to_mask = []
        ## reverse position
        rp = len(alignment)
        for fp,c in enumerate(alignment):
            rp -= 1
            if c == "-": continue
            y = map_mali2genome.mapRowToCol( fp ) - match.mSbjctFrom
            if y < 0: continue
            if quality_scores[y] < options.quality_threshold:
                if is_negative: p = rp
                else: p = fp
                E.debug( "low quality base: id=%s, mali=%i, char=%s, contig=%s, strand=%s, pos=%i, quality=%i" % \
                             (cluster_id, p, c, match.mSbjctId, match.strand, map_mali2genome.mapRowToCol( fp ), quality_scores[y] ) )
                if options.frame > 1:
                    start = (p // options.frame) * options.frame
                    to_mask.extend( list( range(start, start + options.frame) ) )
                else:
                    to_mask.append( p ) 

        regions = Iterators.group_by_distance( sorted(to_mask) )
            
        for start,end in regions:
            options.stdout.write( "%s\t%i\t%i\n" % (cluster_id, start, end ) )

        noutput += 1

    E.info( "ninput=%i, noutput=%i, nmissed=%i" % (ninput, noutput, nmissed) )

    E.Stop()
    
if __name__ == "__main__":
    sys.exit(main())



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
gff_ensembl2gbrowser.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Example::

   python gff_ensembl2gbrowser.py --help

Type::

   python gff_ensembl2gbrowser.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import getopt
import time
import sets
import optparse
import math
import tempfile

import CGAT.Experiment as E

USAGE="""

convert an ensembl gff file into a gbrowser gff file for upload.

Input:
X       ensembl CDS     60708731        60709127        .       -       0        gene_id "ENSMODG00000009707"; transcript_id "ENSMODT00000038508"; exon_number "4"; protein_id "ENSMODP00000036912";
Output:
chr2    gpipe   gene    211249748       211254644       .       -       .       Id=ENSMODG00000009707
chr3    gpipe   mRNA    362769650       362870268       45.24   -       .       Id=ENSMODG00000009707.ENSMODT00000038508 ; Parent=ENSMODG00000009707
chr8    gpipe   CDS     202417740       202417805       59.0909090909   -       0       ID=ENSMODG00000009707.ENSMODT00000038508.4 ; Parent=ENSMODG00000009707.ENSMODT00000038508

"""

parser = E.OptionParser( version = "%prog version: $Id: gff_ensembl2gbrowser.py 2781 2009-09-10 11:33:14Z andreas $")

if __name__ == "__main__":

    parser.add_option( "-s", "--source-id", dest="source_id", type="string",
                       help="source_id to use." )
    
    parser.add_option( "-f", "--feature", dest="feature", type="string",
                       help="name of feature (CDS, exon, ...)." )

    parser.set_defaults(
        source_id = "ensembl",
        feature = "CDS",
        )

    (options, args) = E.Start( parser, add_csv_options  = True)

    input_gffs = GTF.readFromFile( sys.stdin, separator = " " )

    ## sort by genes
    input_gffs.sort( lambda x,y: cmp( (x["gene_id"], x["transcript_id"], x.start), (y["gene_id"], y["transcript_id"], y.start) ) )

    new_gffs = []

    gene_from = None
    gene_to = None
    last_g = None
    mrna_from = None
    mrna_to = None    
    last_mrna = None

    for g in input_gffs:

        if g.feature != options.feature: continue

        if last_g:
            if last_g["transcript_id"] != g["transcript_id"]:
                n = GTF.Entry()
                n.Fill(last_g)
                n.clearAttributes()                
                n.feature = "mRNA"
                n.start = mrna_from
                n.end = mrna_to
                n.frame = "."
                n.addAttribute( "mRNA", "%s.%s" % (last_g["gene_id"], last_g["transcript_id"]), separator = " " )
                n.addAttribute( "Gene", "%s" % (last_g["gene_id"]), separator = " ")
                new_gffs.append( n )
                mrna_from, mrna_to = g.start, g.end
                
            if last_g["gene_id"] != g["gene_id"]:
                n = GTF.Entry()
                n.Fill(last_g)
                n.clearAttributes()
                n.feature = "gene"                
                n.start = gene_from
                n.end = gene_to
                n.frame = "."
                new_gffs.append( n )
                gene_from, gene_to = g.start, g.end
                n.addAttribute( "Gene", "%s" % (last_g["gene_id"]), separator = " " )
                
        else:
            gene_from, gene_to = g.start, g.end
            mrna_from, mrna_to = gene_from, gene_to
                
        n = GTF.Entry()            

        n.Fill( g )
        n.clearAttributes()
        
        n.addAttribute( "mRNA", "%s.%s" % (g["gene_id"], g["transcript_id"] ), separator = " ")

        new_gffs.append( n )

        gene_from, gene_to = min( g.start, gene_from), max(g.end, gene_to)
        mrna_from, mrna_to = min( g.start, mrna_from), max(g.end, mrna_to)        

        last_g = g
        
    ## combine gffs to genes and dump:
    
    for n in new_gffs:
        n.source = options.source_id
        options.stdout.write(str(n) + "\n")
        
    E.Stop()
